@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: write_block_read_column.c 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -17,7 +17,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * This program writes a series of 2D variables with a block-block data
  * partitioning pattern. The 2D variables are read back in a *-block pattern.
- * 
+ *
  * The compile and run commands are given below, together with an ncmpidump of
  * the output file. In this example, NVARS = 4.
  *
@@ -320,11 +320,11 @@ static void
 usage(char *argv0)
 {
     char *help =
-    "Usage: %s [-h | -q] len file_name\n"
+    "Usage: %s [-h] | [-q] [-l len] [file_name]\n"
     "       [-h] Print help\n"
     "       [-q] Quiet mode\n"
-    "       len: local variable of size len x len (default 10)\n"
-    "       filename: output netCDF file name (default ./testfile.nc)\n";
+    "       [-l len]: local variable of size len x len (default 10)\n"
+    "       [filename]: output netCDF file name (default ./testfile.nc)\n";
     fprintf(stderr, help, argv0);
 }
 
@@ -334,7 +334,7 @@ int main(int argc, char** argv) {
     char filename[256];
     int i, rank, nprocs, verbose=1, nerrs=0;
     double timing[10], max_t[10];
-    MPI_Offset len, w_size, r_size, sum_w_size, sum_r_size;
+    MPI_Offset len=0, w_size, r_size, sum_w_size, sum_r_size;
     MPI_Comm comm=MPI_COMM_WORLD;
     MPI_Info w_info_used, r_info_used;
 
@@ -343,23 +343,21 @@ int main(int argc, char** argv) {
     MPI_Comm_size(comm, &nprocs);
 
     /* get command-line arguments */
-    while ((i = getopt(argc, argv, "hq")) != EOF)
+    while ((i = getopt(argc, argv, "hql:")) != EOF)
         switch(i) {
             case 'q': verbose = 0;
                       break;
+            case 'l': len = atoi(optarg);
+                      break;
             case 'h':
             default:  if (rank==0) usage(argv[0]);
                       MPI_Finalize();
                       return 1;
         }
-    argc -= optind;
-    argv += optind;
+    if (argv[optind] == NULL) strcpy(filename, "testfile.nc");
+    else                      snprintf(filename, 256, "%s", argv[optind]);
 
-    len = 10;
-    if (argc > 0) len = strtoll(argv[0],NULL,10);
     len = (len <= 0) ? 10 : len;
-    if (argc == 2) snprintf(filename, 256, "%s", argv[1]);
-    else           strcpy(filename, "testfile.nc");
 
     nerrs += benchmark_write(filename, len, &w_size, &w_info_used, timing);
     nerrs += benchmark_read (filename, len, &r_size, &r_info_used, timing+5);
