@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: block_cyclic.c 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * This example, generalized from column_wise.c, makes a number of nonblocking
@@ -15,7 +15,7 @@
  * block_len. In this example, block_len is set to 2. Blocks are layout in a
  * cyclic fashion in the file. This example can test if PnetCDF can coalesce
  * file offsets and lengths when constructing a merged filetype.
- * 
+ *
  * The compile and run commands are given below, together with an ncmpidump of
  * the output file. In this example, block_len = 2.
  *
@@ -102,6 +102,7 @@ int main(int argc, char** argv) {
     MPI_Offset myNX, G_NX, myOff, block_start, block_len;
     MPI_Offset start[2], count[2];
     MPI_Info info;
+    int bb_enabled=0;
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -117,11 +118,8 @@ int main(int argc, char** argv) {
                       MPI_Finalize();
                       return 1;
         }
-    argc -= optind;
-    argv += optind;
-
-    if (argc == 1) snprintf(filename, 256, "%s", argv[0]);
-    else           strcpy (filename, "testfile.nc");
+    if (argv[optind] == NULL) strcpy(filename, "testfile.nc");
+    else                      snprintf(filename, 256, "%s", argv[optind]);
 
     /* set an MPI-IO hint to disable file offset alignment for fixed-size
      * variables */
@@ -133,6 +131,18 @@ int main(int argc, char** argv) {
     err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, info, &ncid);
     ERR
 
+    {
+        int flag;
+        char hint[MPI_MAX_INFO_VAL];
+        MPI_Info infoused;
+
+        ncmpi_inq_file_info(ncid, &infoused);
+        MPI_Info_get(infoused, "nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
+        if (flag && strcasecmp(hint, "enable") == 0)
+            bb_enabled = 1;
+        MPI_Info_free(&infoused);
+    }
+
     MPI_Info_free(&info);
 
     /* the global array is NY * (NX * nprocs) */
@@ -159,6 +169,11 @@ int main(int argc, char** argv) {
     err = ncmpi_put_vara_int_all(ncid, varid, start, count, buf[0]);
     free(buf[0]);
 
+    if (bb_enabled) {
+        err = ncmpi_flush(ncid);
+        ERR
+    }
+
     /* initialize the buffer with rank ID. Also make the case interesting,
        by allocating buffers separately */
     for (i=0; i<myNX; i++) {
