@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: bput_varn_uint.c 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * This example tests nonblocking buffered write varn APIs, including
@@ -178,10 +178,8 @@ int main(int argc, char** argv)
                       MPI_Finalize();
                       return 1;
         }
-    argc -= optind;
-    argv += optind;
-    if (argc == 1) snprintf(filename, 256, "%s", argv[0]);
-    else           strcpy(filename, "testfile.nc");
+    if (argv[optind] == NULL) strcpy(filename, "testfile.nc");
+    else                      snprintf(filename, 256, "%s", argv[optind]);
 
     if (nprocs != 4 && rank == 0 && verbose)
         printf("Warning: %s is intended to run on 4 processes\n",exec);
@@ -218,10 +216,10 @@ int main(int argc, char** argv)
     starts[n][2][0]=2; starts[n][2][1]=6; counts[n][2][0]=1; counts[n][2][1]=2;
     starts[n][3][0]=3; starts[n][3][1]=0; counts[n][3][0]=1; counts[n][3][1]=3;
     /* starts[n][][] n_counts[n][][] indicate the following: ("-" means skip)
-              -  -  -  -  -  X  X  -  -  - 
-              X  -  -  -  -  -  -  -  -  - 
-              -  -  -  -  -  -  X  X  -  - 
-              X  X  X  -  -  -  -  -  -  - 
+              -  -  -  -  -  X  X  -  -  -
+              X  -  -  -  -  -  -  -  -  -
+              -  -  -  -  -  -  X  X  -  -
+              X  X  X  -  -  -  -  -  -  -
      */
     n = (rank+1) % 4;
     num_segs[n] = 6; /* number of segments for this request */
@@ -232,10 +230,10 @@ int main(int argc, char** argv)
     starts[n][4][0]=2; starts[n][4][1]=8; counts[n][4][0]=1; counts[n][4][1]=2;
     starts[n][5][0]=3; starts[n][5][1]=4; counts[n][5][0]=1; counts[n][5][1]=3;
     /* starts[n][][] counts[n][][] indicate the following pattern.
-              -  -  -  X  X  -  -  -  X  X 
-              -  -  -  -  -  X  X  -  -  - 
-              X  X  -  -  -  -  -  -  X  X 
-              -  -  -  -  X  X  X  -  -  - 
+              -  -  -  X  X  -  -  -  X  X
+              -  -  -  -  -  X  X  -  -  -
+              X  X  -  -  -  -  -  -  X  X
+              -  -  -  -  X  X  X  -  -  -
      */
     n = (rank+2) % 4;
     num_segs[n] = 5; /* number of segments for this request */
@@ -245,10 +243,10 @@ int main(int argc, char** argv)
     starts[n][3][0]=2; starts[n][3][1]=2; counts[n][3][0]=1; counts[n][3][1]=1;
     starts[n][4][0]=3; starts[n][4][1]=3; counts[n][4][0]=1; counts[n][4][1]=1;
     /* starts[n][][] counts[n][][] indicate the following pattern.
-              -  -  -  -  -  -  -  X  -  - 
-              -  X  X  X  -  -  -  X  X  X 
-              -  -  X  -  -  -  -  -  -  - 
-              -  -  -  X  -  -  -  -  -  - 
+              -  -  -  -  -  -  -  X  -  -
+              -  X  X  X  -  -  -  X  X  X
+              -  -  X  -  -  -  -  -  -  -
+              -  -  -  X  -  -  -  -  -  -
      */
     n = (rank+3) % 4;
     num_segs[n] = 4; /* number of segments for this request */
@@ -257,10 +255,10 @@ int main(int argc, char** argv)
     starts[n][2][0]=2; starts[n][2][1]=3; counts[n][2][0]=1; counts[n][2][1]=3;
     starts[n][3][0]=3; starts[n][3][1]=7; counts[n][3][0]=1; counts[n][3][1]=3;
      /*starts[n][][] counts[n][][] indicate the following pattern.
-              X  X  X  -  -  -  -  -  -  - 
-              -  -  -  -  X  -  -  -  -  - 
-              -  -  -  X  X  X  -  -  -  - 
-              -  -  -  -  -  -  -  X  X  X 
+              X  X  X  -  -  -  -  -  -  -
+              -  -  -  -  X  -  -  -  -  -
+              -  -  -  X  X  X  -  -  -  -
+              -  -  -  -  -  -  -  X  X  X
      */
 
     /* only rank 0, 1, 2, and 3 do I/O:
