@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: nonblocking_write.c 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 /*    This example is similar to collective_write.c but using nonblocking APIs.
  *    It creates a netcdf file in CD-5 format and writes a number of
@@ -48,11 +48,11 @@ static void
 usage(char *argv0)
 {
     char *help =
-    "Usage: %s [-h] | [-q] [file_name] [len]\n"
+    "Usage: %s [-h] | [-q] [-l len] [file_name]\n"
     "       [-h] Print help\n"
     "       [-q] Quiet mode (reports when fail)\n"
-    "       [filename] output netCDF file name\n"
-    "       [len] size of each dimension of the local array\n";
+    "       [-l len] size of each dimension of the local array\n"
+    "       [filename] output netCDF file name\n";
     fprintf(stderr, help, argv0);
 }
 
@@ -85,7 +85,7 @@ int main(int argc, char **argv)
 {
     extern int optind;
     int i, j, verbose=1, err, nerrs=0;
-    int nprocs, len, *buf[NUM_VARS], bufsize, rank;
+    int nprocs, len=0, *buf[NUM_VARS], bufsize, rank;
     int gsizes[NDIMS], psizes[NDIMS];
     double write_timing, max_write_timing, write_bw;
     char filename[256], str[512];
@@ -99,22 +99,20 @@ int main(int argc, char **argv)
     MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
 
     /* get command-line arguments */
-    while ((i = getopt(argc, argv, "hq")) != EOF)
+    while ((i = getopt(argc, argv, "hql:")) != EOF)
         switch(i) {
             case 'q': verbose = 0;
                       break;
+            case 'l': len = atoi(optarg);
+                      break;
             case 'h':
             default:  if (rank==0) usage(argv[0]);
                       MPI_Finalize();
                       return 1;
         }
-    argc -= optind;
-    argv += optind;
-    if (argc >= 1) snprintf(filename, 256, "%s", argv[0]);
-    else           strcpy(filename, "testfile.nc");
+    if (argv[optind] == NULL) strcpy(filename, "testfile.nc");
+    else                      snprintf(filename, 256, "%s", argv[optind]);
 
-    len = 10; 
-    if (argc >= 2) len = (int)strtol(argv[1],NULL,10); /* optional argument */
     len = (len <= 0) ? 10 : len;
 
     for (i=0; i<NDIMS; i++) psizes[i] = 0;
