@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: column_wise.cpp 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * This example makes a number of nonblocking API calls, each writes a single
@@ -19,8 +19,8 @@
  * pairs, merging the pairs across multiple nonblocking calls, and sorting
  * them into an increasing order. The sorted pairs are used to construct a
  * fileview that meets the monotonically non-decreasing offset requirement,
- * and thus the nonblocking requests can be serviced by a single MPI-IO call. 
- * 
+ * and thus the nonblocking requests can be serviced by a single MPI-IO call.
+ *
  * The compile and run commands are given below, together with an ncmpidump of
  * the output file.
  *
@@ -108,10 +108,8 @@ int main(int argc, char** argv)
                       MPI_Finalize();
                       return 1;
         }
-    argc -= optind;
-    argv += optind;
-    if (argc == 1) snprintf(filename, 256, "%s", argv[0]);
-    else           strcpy(filename, "testfile.nc");
+    if (argv[optind] == NULL) strcpy(filename, "testfile.nc");
+    else                      snprintf(filename, 256, "%s", argv[optind]);
 
     try {
         NcmpiFile nc(MPI_COMM_WORLD, filename, NcmpiFile::replace,
