@@ -2,7 +2,7 @@
  *  Copyright (C) 2017, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: file.c 3590 2017-12-12 06:49:32Z wkliao $ */
+/* $Id$ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -44,7 +44,7 @@ static int ncmpi_default_create_format = NC_FORMAT_CLASSIC;
 /* get a new ID from the PNC list */
 static int
 new_id_PNCList(int *new_id)
-{   
+{
     int i;
 
     *new_id = -1;
@@ -64,7 +64,7 @@ new_id_PNCList(int *new_id)
 static int
 add_to_PNCList(PNC *pncp,
                int  new_id)
-{   
+{
     /* these checks below are redundant */
     assert(pncp != NULL);
     assert(new_id >= 0);
@@ -234,7 +234,7 @@ ncmpi_create(MPI_Comm    comm,
              int        *ncidp)
 {
     int default_format, rank, status=NC_NOERR, err;
-    int safe_mode=0, mpireturn, root_cmode;
+    int safe_mode=0, mpireturn, root_cmode, relax_coord_bound;
     char *env_str;
     MPI_Info combined_info;
     void *ncp;
@@ -243,13 +243,17 @@ ncmpi_create(MPI_Comm    comm,
 #ifdef BUILD_DRIVER_FOO
     int enable_foo_driver=0;
 #endif
+#ifdef BUILD_DRIVER_BB
+    int enable_bb_driver=0;
+#endif
 
     MPI_Comm_rank(comm, &rank);
 
 #ifdef PNETCDF_DEBUG
     safe_mode = 1;
-    /* this configure time setting will be overwritten by the run-time
-     * environment variable PNETCDF_SAFE_MODE */
+    /* When debug mode is enabled at the configure time, safe_mode is by
+     * default enabled. This can be overwritten by the run-time environment
+     * variable PNETCDF_SAFE_MODE */
 #endif
     /* get environment variable PNETCDF_SAFE_MODE
      * if it is set to 1, then we perform a strict parameter consistent test
@@ -261,6 +265,22 @@ ncmpi_create(MPI_Comm    comm,
          * be '\0' (null character). In this case, safe_mode is enabled */
     }
 
+    /* get environment variable PNETCDF_RELAX_COORD_BOUND
+     * if it is set to 0, then we perform a strict start bound check
+     */
+#ifndef RELAX_COORD_BOUND
+    relax_coord_bound = 0;
+#else
+    relax_coord_bound = 1;
+#endif
+    if ((env_str = getenv("PNETCDF_RELAX_COORD_BOUND")) != NULL) {
+        if (*env_str == '0') relax_coord_bound = 0;
+        else                 relax_coord_bound = 1;
+        /* if PNETCDF_RELAX_COORD_BOUND is set but without a value, *env_str
+         * can be '\0' (null character). This is equivalent to setting
+         * relax_coord_bound to 1 */
+    }
+
     /* path's validity is checked in MPI-IO with error code MPI_ERR_BAD_FILE
      * path consistency is checked in MPI-IO with error code MPI_ERR_NOT_SAME
      */
@@ -285,29 +305,47 @@ ncmpi_create(MPI_Comm    comm,
     /* continue to use root's cmode to create the file, but will report cmode
      * inconsistency error, if there is any */
 
-    /* combine user's info and PNETCDF_HINTS env variable */
+    /* combine user's MPI info and PNETCDF_HINTS env variable */
     combine_env_hints(info, &combined_info);
 
 #ifdef BUILD_DRIVER_FOO
-    /* check if nc_foo_driver is enabled */
     if (combined_info != MPI_INFO_NULL) {
         char value[MPI_MAX_INFO_VAL];
         int flag;
 
+        /* check if nc_foo_driver is enabled */
         MPI_Info_get(combined_info, "nc_foo_driver", MPI_MAX_INFO_VAL-1,
                      value, &flag);
         if (flag && strcasecmp(value, "enable") == 0)
             enable_foo_driver = 1;
     }
+#endif
+#ifdef BUILD_DRIVER_BB
+    if (combined_info != MPI_INFO_NULL) {
+        char value[MPI_MAX_INFO_VAL];
+        int flag;
 
+        /* check if nc_burst_buf is enabled */
+        MPI_Info_get(combined_info, "nc_burst_buf", MPI_MAX_INFO_VAL-1,
+                     value, &flag);
+        if (flag && strcasecmp(value, "enable") == 0)
+            enable_bb_driver = 1;
+    }
+#endif
+
+    /* Use environment variable and cmode to tell the file format
+     * which is later used to select the right driver.
+     */
+#ifdef BUILD_DRIVER_FOO
     if (enable_foo_driver)
         driver = ncfoo_inq_driver();
     else
 #endif
-        /* TODO: Use environment variable and cmode to tell the file format
-         * which is later used to select the right driver. For now, we have
-         * only one driver, ncmpio.
-         */
+#ifdef BUILD_DRIVER_BB
+    if (enable_bb_driver)
+        driver = ncbbio_inq_driver();
+    else
+#endif
         driver = ncmpio_inq_driver();
 
 #if 0 /* refer to netCDF library's USE_REFCOUNT */
@@ -353,8 +391,14 @@ ncmpi_create(MPI_Comm    comm,
     pncp->flag       = NC_MODE_DEF | NC_MODE_CREATE;
     pncp->ncp        = ncp;
 
-    if (safe_mode)         pncp->flag |= NC_MODE_SAFE;
+    if (safe_mode)          pncp->flag |= NC_MODE_SAFE;
+    if (!relax_coord_bound) pncp->flag |= NC_MODE_STRICT_COORD_BOUND;
     /* if (enable_foo_driver) pncp->flag |= NC_MODE_BB; */
+
+    /* Duplicate comm, because users may free it. Note MPI_Comm_dup is
+     * collective. pncp->comm will be passed to drivers, so there is no need
+     * for a driver to duplicate it again.
+     */
     MPI_Comm_dup(comm, &pncp->comm);
 
     /* set the file format version based on the create mode, cmode */
@@ -390,7 +434,7 @@ ncmpi_open(MPI_Comm    comm,
            int        *ncidp)  /* OUT */
 {
     int i, nalloc, rank, format, msg[2], status=NC_NOERR, err;
-    int safe_mode=0, mpireturn, root_omode;
+    int safe_mode=0, mpireturn, root_omode, relax_coord_bound;
     char *env_str;
     MPI_Info combined_info;
     void *ncp;
@@ -399,13 +443,17 @@ ncmpi_open(MPI_Comm    comm,
 #ifdef BUILD_DRIVER_FOO
     int enable_foo_driver=0;
 #endif
+#ifdef BUILD_DRIVER_BB
+    int enable_bb_driver=0;
+#endif
 
     MPI_Comm_rank(comm, &rank);
 
 #ifdef PNETCDF_DEBUG
     safe_mode = 1;
-    /* this configure time setting will be overwritten by the run-time
-     * environment variable PNETCDF_SAFE_MODE */
+    /* When debug mode is enabled at the configure time, safe_mode is by
+     * default enabled. This can be overwritten by the run-time environment
+     * variable PNETCDF_SAFE_MODE */
 #endif
     /* get environment variable PNETCDF_SAFE_MODE
      * if it is set to 1, then we perform a strict parameter consistent test
@@ -417,6 +465,22 @@ ncmpi_open(MPI_Comm    comm,
          * be '\0' (null character). In this case, safe_mode is enabled */
     }
 
+    /* get environment variable PNETCDF_RELAX_COORD_BOUND
+     * if it is set to 0, then we perform a strict start bound check
+     */
+#ifndef RELAX_COORD_BOUND
+    relax_coord_bound = 0;
+#else
+    relax_coord_bound = 1;
+#endif
+    if ((env_str = getenv("PNETCDF_RELAX_COORD_BOUND")) != NULL) {
+        if (*env_str == '0') relax_coord_bound = 0;
+        else                 relax_coord_bound = 1;
+        /* if PNETCDF_RELAX_COORD_BOUND is set but without a value, *env_str
+         * can be '\0' (null character). This is equivalent to setting
+         * relax_coord_bound to 1 */
+    }
+
     /* path's validity is checked in MPI-IO with error code MPI_ERR_BAD_FILE
      * path consistency is checked in MPI-IO with error code MPI_ERR_NOT_SAME
      */
@@ -462,43 +526,63 @@ ncmpi_open(MPI_Comm    comm,
     /* continue to use root's omode to open the file, but will report omode
      * inconsistency error, if there is any */
 
-    /* combine user's info and PNETCDF_HINTS env variable */
+    /* combine user's MPI info and PNETCDF_HINTS env variable */
     combine_env_hints(info, &combined_info);
 
 #ifdef BUILD_DRIVER_FOO
-    /* check if nc_foo_driver is enabled */
     if (combined_info != MPI_INFO_NULL) {
         char value[MPI_MAX_INFO_VAL];
         int flag;
 
+        /* check if nc_foo_driver is enabled */
         MPI_Info_get(combined_info, "nc_foo_driver", MPI_MAX_INFO_VAL-1,
                      value, &flag);
         if (flag && strcasecmp(value, "enable") == 0)
             enable_foo_driver = 1;
-    }
 
+    }
+#endif
+#ifdef BUILD_DRIVER_BB
+    if (combined_info != MPI_INFO_NULL) {
+        char value[MPI_MAX_INFO_VAL];
+        int flag;
+
+        /* check if nc_burst_buf is enabled */
+        MPI_Info_get(combined_info, "nc_burst_buf", MPI_MAX_INFO_VAL-1,
+                     value, &flag);
+        if (flag && strcasecmp(value, "enable") == 0)
+            enable_bb_driver = 1;
+    }
+#endif
+
+#ifdef BUILD_DRIVER_FOO
     if (enable_foo_driver)
         driver = ncfoo_inq_driver();
     else
 #endif
-        /* TODO: currently we only have ncmpio driver. Need to add other
-         * drivers once they are available
-         */
+#ifdef BUILD_DRIVER_BB
+    if (enable_bb_driver)
+        driver = ncbbio_inq_driver();
+    else
+#endif
+    {
+        /* ncmpio driver */
         if (format == NC_FORMAT_CLASSIC ||
             format == NC_FORMAT_CDF2 ||
             format == NC_FORMAT_CDF5) {
-        driver = ncmpio_inq_driver();
-    }
-    else if (format == NC_FORMAT_NETCDF4_CLASSIC) {
-        fprintf(stderr,"NC_FORMAT_NETCDF4_CLASSIC is not yet supported\n");
-        DEBUG_RETURN_ERROR(NC_ENOTSUPPORT)
-    }
-    else if (format == NC_FORMAT_NETCDF4) {
-        fprintf(stderr,"NC_FORMAT_NETCDF4 is not yet supported\n");
-        DEBUG_RETURN_ERROR(NC_ENOTSUPPORT)
-    }
-    else { /* unrecognized file format */
-        DEBUG_RETURN_ERROR(NC_ENOTNC)
+            driver = ncmpio_inq_driver();
+        }
+        else if (format == NC_FORMAT_NETCDF4_CLASSIC) {
+            fprintf(stderr,"NC_FORMAT_NETCDF4_CLASSIC is not yet supported\n");
+            DEBUG_RETURN_ERROR(NC_ENOTSUPPORT)
+        }
+        else if (format == NC_FORMAT_NETCDF4) {
+            fprintf(stderr,"NC_FORMAT_NETCDF4 is not yet supported\n");
+            DEBUG_RETURN_ERROR(NC_ENOTSUPPORT)
+        }
+        else { /* unrecognized file format */
+            DEBUG_RETURN_ERROR(NC_ENOTNC)
+        }
     }
 
     /* get a new ID from NCPList */
@@ -541,9 +625,16 @@ ncmpi_open(MPI_Comm    comm,
     pncp->flag       = 0;
     pncp->ncp        = ncp;
     pncp->format     = format;
+
     if (!fIsSet(omode, NC_WRITE)) pncp->flag |= NC_MODE_RDONLY;
     if (safe_mode)                pncp->flag |= NC_MODE_SAFE;
+    if (!relax_coord_bound)       pncp->flag |= NC_MODE_STRICT_COORD_BOUND;
     /* if (enable_foo_driver)        pncp->flag |= NC_MODE_BB; */
+
+    /* Duplicate comm, because users may free it. Note MPI_Comm_dup is
+     * collective. pncp->comm will be passed to drivers, so there is no need
+     * for a driver to duplicate it again.
+     */
     MPI_Comm_dup(comm, &pncp->comm);
 
     /* add to the PNCList */
@@ -573,7 +664,7 @@ ncmpi_open(MPI_Comm    comm,
         int ndims;
         err = driver->inq_var(pncp->ncp, i, NULL, &xtype, &ndims,
                               NULL, NULL, NULL, NULL, NULL);
-        if (err != NC_NOERR) return err;
+        if (err != NC_NOERR) goto fn_exit;
         pncp->vars[i].xtype  = xtype;
         pncp->vars[i].ndims  = ndims;
         pncp->vars[i].recdim = -1;   /* if fixed-size variable */
@@ -682,7 +773,7 @@ ncmpi__enddef(int        ncid,
               MPI_Offset v_align,
               MPI_Offset v_minfree,
               MPI_Offset r_align)
-{   
+{
     int err=NC_NOERR;
     PNC *pncp;
 
@@ -761,10 +852,10 @@ ncmpi_redef(int ncid)
         DEBUG_RETURN_ERROR(NC_EPERM)
     /* if open mode is inconsistent, then this return might cause parallel
      * program to hang */
-    
+
     /* cannot be in define mode, must enter from data mode */
     if (fIsSet(pncp->flag, NC_MODE_DEF)) DEBUG_RETURN_ERROR(NC_EINDEFINE)
-    
+
     /* calling the subroutine that implements ncmpi_redef() */
     err = pncp->driver->redef(pncp->ncp);
     if (err != NC_NOERR) return err;
@@ -791,6 +882,24 @@ ncmpi_sync(int ncid)
     return pncp->driver->sync(pncp->ncp);
 }
 
+/*----< ncmpi_flush() >-------------------------------------------------------*/
+/* This API is a collective subroutine, and must be called in data mode, no
+ * matter if it is in collective or independent data mode.
+ */
+int
+ncmpi_flush(int ncid)
+{
+    int err;
+    PNC *pncp;
+
+    /* check if ncid is valid */
+    err = PNC_check_id(ncid, &pncp);
+    if (err != NC_NOERR) return err;
+
+    /* calling the subroutine that implements ncmpi_flush() */
+    return pncp->driver->flush(pncp->ncp);
+}
+
 /*----< ncmpi_abort() >------------------------------------------------------*/
 /* This is a collective subroutine. */
 int
@@ -1011,7 +1120,7 @@ int
 ncmpi_inq_path(int   ncid,
                int  *pathlen,/* Ignored if NULL */
                char *path)   /* must have already been allocated. Ignored if NULL */
-{        
+{
     int err;
     PNC *pncp;
 
@@ -1444,7 +1553,7 @@ ncmpi_buffer_detach(int ncid)
 }
 
 /*----< ncmpi_delete() >-----------------------------------------------------*/
-/* 
+/*
  * filename: the name of the file we will remove.
  * info: MPI info object, in case underlying file system needs hints.
  *
