@@ -2,7 +2,7 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: ncmpio_NC.h 3583 2017-12-10 17:27:53Z wkliao $ */
+/* $Id$ */
 
 #ifndef _NC_H
 #define _NC_H
@@ -20,9 +20,9 @@
 #define FILE_ALIGNMENT_DEFAULT 512
 #define FILE_ALIGNMENT_LB      4
 
-/* define MPI_OFFSET if not defined */
+/* MPI_OFFSET datatype was introduced in MPI 2.2 */
 #ifndef HAVE_DECL_MPI_OFFSET
-    #ifdef HAVE_DECL_MPI_LONG_LONG_INT
+    #if SIZEOF_MPI_OFFSET ==  SIZEOF_MPI_LONG_LONG_INT
         #define MPI_OFFSET MPI_LONG_LONG_INT
     #else
         #define MPI_OFFSET MPI_INT
@@ -269,35 +269,41 @@ ncmpio_NC_lookupvar(NC *ncp, int varid, NC_var **varp);
 /*
  *  The PnetCDF non-blocking I/O request type
  */
-#define NC_BUFTYPE_IS_CONTIG  0x00000001
-#define NC_NEED_SWAP_BACK_BUF 0x00000002
+#define NC_REQ_TO_FREE             0x00000001
+#define NC_REQ_SKIP                0x00000002
+#define NC_REQ_STRIDE_NULL         0x00000004
+#define NC_REQ_BUF_TYPE_IS_CONTIG  0x00000008
+#define NC_REQ_BUF_TYPE_CONVERT    0x00000010
+#define NC_REQ_BUF_BYTE_SWAP       0x00000020
+#define NC_REQ_XBUF_TO_BE_FREED    0x00000040
+
+typedef struct NC_lead_req {
+    int           flag;         /* bit-wise OR of the above NC_REQ_* flags */
+    int           id;           /* even number for write, odd for read */
+    int           nonlead_off;  /* start index in the non-lead queue */
+    int           nonlead_num;  /* number of non-lead requests */
+    int           abuf_index;   /* index in the abuf occupy_table. -1 means not
+                                   using attached buffer */
+    void         *buf;          /* user buffer */
+    void         *xbuf;         /* buffer in external type, may be == buf */
+    NC_var       *varp;         /* pointer to NC variable object */
+    MPI_Offset    nelems;       /* total number of array elements requested */
+    MPI_Offset    max_rec;      /* highest record requested */
+    MPI_Offset    bufcount;     /* number of buftype in this request */
+    MPI_Offset   *start;        /* [varp->ndims*3] for start/count/stride */
+    MPI_Datatype  buftype;      /* user defined derived data type */
+    MPI_Datatype  itype;        /* internal element data type in buftype */
+    MPI_Datatype  imaptype;     /* derived data type constructed from imap */
+    int          *status;       /* pointer to user's status */
+} NC_lead_req;
+
 typedef struct NC_req {
-    int            flag;        /* BUFTYPE_IS_CONTIG | NEED_SWAP_BACK_BUF */
-    int            id;          /* even number for write, odd for read */
-    int            abuf_index;  /* index in the abuf occupy_table
-                                   -1 means not using attached buffer */
-    MPI_Offset     num_recs;    /* number of records requested (1 for
-                                   fixed-size variable) */
-    void          *buf;         /* the original user buffer */
-    void          *xbuf;        /* the buffer used to read/write, may point to
-                                   the same address as buf */
-    void          *tmpBuf;      /* tmp buffer to be freed, used only by
-                                   nonblocking varn when buftype is noncontig */
-    void          *userBuf;     /* user buffer to be unpacked from tmpBuf. used
-                                   only by by nonblocking varn when buftype is
-                                   noncontig */
-    NC_var        *varp;
-    MPI_Offset    *start;        /* [varp->ndims] */
-    MPI_Offset    *count;        /* [varp->ndims] */
-    MPI_Offset    *stride;       /* [varp->ndims] */
-    MPI_Offset     bnelems;      /* number of elements in user buffer */
-    MPI_Offset     offset_start; /* starting of aggregate access region */
-    MPI_Offset     offset_end;   /*   ending of aggregate access region */
-    MPI_Offset     bufcount;     /* the number of buftype in this request */
-    MPI_Datatype   buftype;      /* user defined derived data type */
-    MPI_Datatype   ptype;        /* element data type in buftype */
-    MPI_Datatype   imaptype;     /* derived data type constructed from imap */
-    int           *status;       /* pointer to user's status */
+    MPI_Offset    offset_start; /* starting offset of aggregate access region */
+    MPI_Offset    offset_end;   /*   ending offset of aggregate access region */
+    MPI_Offset    nelems;       /* number of array elements requested */
+    MPI_Offset   *start;        /* [varp->ndims*3] for start/count/stride */
+    void         *xbuf;         /* buffer in external type, used in file I/O calls */
+    NC_lead_req  *lead;         /* point to lead request */
 } NC_req;
 
 #define NC_ABUF_DEFAULT_TABLE_SIZE 128
@@ -334,12 +340,11 @@ struct NC {
                                  * collective open to independent open */
     int           format;       /* 1, 2, or 5 corresponding to CDF-1, 2, or 5 */
     int           safe_mode;    /* 0 or 1, for parameter consistency check */
-    int           numGetReqs;   /* number of pending nonblocking get requests */
-    int           numPutReqs;   /* number of pending nonblocking put requests */
 #ifdef ENABLE_SUBFILING
     int           subfile_mode; /* 0 or 1, for disable/enable subfiling */
     int           num_subfiles; /* number of subfiles */
     struct NC    *ncp_sf;       /* ncp of subfile */
+    MPI_Comm      comm_sf;      /* subfile MPI communicator */
 #endif
     int           striping_unit; /* file stripe size of the file */
     int           chunk;       /* chunk size for reading header */
@@ -367,8 +372,16 @@ struct NC {
     NC_attrarray  attrs;    /* global attributes defined */
     NC_vararray   vars;     /* variables defined */
 
-    NC_req       *get_list; /* list of nonblocking read requests */
-    NC_req       *put_list; /* list of nonblocking write requests */
+    int           numLeadGetReqs; /* number of pending lead get requests */
+    int           numLeadPutReqs; /* number of pending lead put requests */
+    NC_lead_req  *get_lead_list;  /* list of lead nonblocking read requests */
+    NC_lead_req  *put_lead_list;  /* list of lead nonblocking write requests */
+
+    int           numGetReqs;   /* number of pending nonblocking get requests */
+    int           numPutReqs;   /* number of pending nonblocking put requests */
+    NC_req       *get_list;     /* list of nonblocking read requests */
+    NC_req       *put_list;     /* list of nonblocking write requests */
+
     NC_buf       *abuf;     /* attached buffer, used by bput APIs */
 
     char         *path;     /* file name */
@@ -442,21 +455,29 @@ ncmpio_write_numrecs(NC *ncp, MPI_Offset new_numrecs);
 extern int
 ncmpio_filetype_create_vars(const NC* ncp, const NC_var* varp,
                 const MPI_Offset start[], const MPI_Offset count[],
-                const MPI_Offset stride[], int rw_flag, int *blocklen,
-                MPI_Offset *offset, MPI_Datatype *filetype,
-                int *is_filetype_contig);
+                const MPI_Offset stride[], MPI_Offset *offset,
+                MPI_Datatype *filetype, int *is_filetype_contig);
 
-extern int                
+extern int
 ncmpio_file_set_view(const NC *ncp, MPI_File fh, MPI_Offset *offset,
                 MPI_Datatype filetype);
 
 /* Begin defined in ncmpio_igetput.m4 ---------------------------------------*/
+extern int
+ncmpio_abuf_malloc(NC *ncp, MPI_Offset nbytes, void **buf, int *abuf_index);
+
+extern int
+ncmpio_abuf_dealloc(NC *ncp, int abuf_index);
+
+extern int
+ncmpio_add_record_requests(NC_req *reqs, MPI_Offset num_recs,
+                           const MPI_Offset *stride);
+
 extern int
 ncmpio_igetput_varm(NC *ncp, NC_var *varp, const MPI_Offset *start,
                 const MPI_Offset *stride, const MPI_Offset *imap,
                 const MPI_Offset *count, void *buf, MPI_Offset bufcount,
-                MPI_Datatype datatype, int *reqid, int reqMode,
-                int isSameGroup);
+                MPI_Datatype datatype, int *reqid, int reqMode);
 
 /* Begin defined in ncmpio_hash_func.c --------------------------------------*/
 extern int
@@ -528,16 +549,21 @@ ncmpio_set_pnetcdf_hints(NC *ncp, MPI_Info info);
 extern int
 ncmpio_NC_check_name(const char *name, int file_ver);
 
+extern int
+ncmpio_first_offset(const NC *ncp, const NC_var *varp, const MPI_Offset start[],
+                    MPI_Offset *offset);
+
 extern int
 ncmpio_last_offset(const NC *ncp, const NC_var *varp, const MPI_Offset starts[],
                    const MPI_Offset counts[], const MPI_Offset strides[],
-                   const int rw_flag, MPI_Offset *offset_ptr);
+                   MPI_Offset *offset_ptr);
 
 extern int
 ncmpio_pack_xbuf(int format, NC_var *varp, MPI_Offset bufcount,
                  MPI_Datatype buftype, int buftype_is_contig, MPI_Offset nelems,
-                 MPI_Datatype etype, MPI_Datatype imaptype, int need_convert,
-                 int need_swap, size_t xbuf_size, void *buf, void *xbuf);
+                 MPI_Datatype etype, int esize, MPI_Datatype imaptype,
+                 int need_convert, int need_swap, size_t xbuf_size, void *buf,
+                 void *xbuf);
 
 extern int
 ncmpio_unpack_xbuf(int format, NC_var *varp, MPI_Offset bufcount,
