@@ -2,7 +2,7 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: ncmpio_create.c 3482 2017-10-25 21:02:12Z wkliao $ */
+/* $Id$ */
 
 /*
  * This file implements the corresponding APIs defined in src/dispatchers/file.c
@@ -41,7 +41,6 @@ ncmpio_create(MPI_Comm     comm,
     char *env_str;
     int rank, mpiomode, err, mpireturn, default_format;
     MPI_File fh;
-    MPI_Comm dup_comm;
     MPI_Info info_used;
     NC *ncp=NULL;
 
@@ -172,17 +171,10 @@ ncmpio_create(MPI_Comm     comm,
         /* reset errno, as MPI_File_open may change it, even for MPI_SUCCESS */
         errno = 0;
 
-    /* duplicate communicator as user may free it later */
-    mpireturn = MPI_Comm_dup(comm, &dup_comm);
-    if (mpireturn != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(mpireturn, "MPI_Comm_dup");
-
     /* get the file info actually used by MPI-IO (may alter user's info) */
     mpireturn = MPI_File_get_info(fh, &info_used);
-    if (mpireturn != MPI_SUCCESS) {
-        MPI_Comm_free(&dup_comm);
+    if (mpireturn != MPI_SUCCESS)
         return ncmpii_error_mpi2nc(mpireturn, "MPI_File_get_info");
-    }
 
     /* Now the file has been successfully created, allocate/set NC object */
 
@@ -221,43 +213,9 @@ ncmpio_create(MPI_Comm     comm,
     /* extract I/O hints from user info */
     ncmpio_set_pnetcdf_hints(ncp, info);
 
-#if 0
-    ncp->safe_mode    = 0;
-    ncp->numGetReqs   = 0; /* number of pending non-blocking get requests */
-    ncp->numPutReqs   = 0; /* number of pending non-blocking put requests */
-    ncp->h_align      = 0; /* value 0 indicates the hint is not set */
-    ncp->v_align      = 0;
-    ncp->r_align      = 0;
-    ncp->h_minfree    = 0;
-    ncp->v_minfree    = 0;
-    ncp->get_list     = NULL;
-    ncp->put_list     = NULL;
-    ncp->abuf         = NULL;
-    ncp->old          = NULL;
-    ncp->put_size     = 0;    /* bytes written so far */
-    ncp->get_size     = 0;    /* bytes read    so far */
-
-#ifndef SEARCH_NAME_LINEARLY
-    for (i=0; i<HASH_TABLE_SIZE; i++) {
-        /* initialize dim name lookup table */
-        ncp->dims.nameT[i].num = 0;
-        ncp->dims.nameT[i].list = NULL;
-        /* initialize var name lookup table */
-        ncp->vars.nameT[i].num = 0;
-        ncp->vars.nameT[i].list = NULL;
-    }
-#endif
-
-#ifdef ENABLE_SUBFILING
-    ncp->subfile_mode = 0;
-    ncp->num_subfiles = 0;
-    ncp->ncp_sf       = NULL; /* pointer to subfile NC object */
-#endif
-#endif
-
     /* For file create, ignore if NC_NOWRITE set in cmode by user */
     ncp->iomode         = cmode | NC_WRITE;
-    ncp->comm           = dup_comm;
+    ncp->comm           = comm;  /* reuse comm duplicated in dispatch layer */
     ncp->mpiinfo        = info_used;
     ncp->mpiomode       = mpiomode;
     ncp->collective_fh  = fh;
