@@ -2,7 +2,7 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: ncmpio_header_get.c 3589 2017-12-12 06:33:02Z wkliao $ */
+/* $Id$ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -627,8 +627,6 @@ hdr_get_NC_name(bufferinfo  *gbp, char **namep)
 #ifdef PNETCDF_DEBUG
             fprintf(stderr,"Error in file %s func %s line %d: NetCDF header corrupted, non-zero padding found\n",__FILE__,__func__,__LINE__);
 #endif
-            NCI_Free(*namep);
-            *namep = NULL;
             DEBUG_ASSIGN_ERROR(err, NC_ENULLPAD) /* not a fatal error */
         }
 #endif
@@ -640,7 +638,7 @@ hdr_get_NC_name(bufferinfo  *gbp, char **namep)
 
 /*----< hdr_get_NC_dim() >---------------------------------------------------*/
 static int
-hdr_get_NC_dim(bufferinfo *gbp, NC_dim **dimpp)
+hdr_get_NC_dim(bufferinfo *gbp, int unlimited_id, NC_dim **dimpp)
 {
     /* netCDF file format:
      *  ...
@@ -672,14 +670,23 @@ hdr_get_NC_dim(bufferinfo *gbp, NC_dim **dimpp)
         err = hdr_get_uint64(gbp, &tmp);
         dim_length = (MPI_Offset)tmp;
     }
-    if (err != NC_NOERR) { /* frees dim */
+    if (err != NC_NOERR) { /* free space allocated for name */
         NCI_Free(name);
         return err;
     }
 
+    /* check if unlimited_id already set */
+    if (unlimited_id != -1 && dim_length == 0) {
+        NCI_Free(name);
+        return NC_EUNLIMIT;
+    }
+
     /* allocate and initialize NC_dim object */
     dimp = (NC_dim*) NCI_Malloc(sizeof(NC_dim));
-    if (dimp == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
+    if (dimp == NULL) {
+        NCI_Free(name);
+        DEBUG_RETURN_ERROR(NC_ENOMEM)
+    }
     dimp->name     = name;
     dimp->name_len = strlen(name);
     dimp->size     = dim_length;
@@ -765,7 +772,7 @@ hdr_get_NC_dimarray(bufferinfo *gbp, NC_dimarray *ncap)
     if (ncap->value == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
 
     for (i=0; i<ndefined; i++) {
-        err = hdr_get_NC_dim(gbp, ncap->value + i);
+        err = hdr_get_NC_dim(gbp, ncap->unlimited_id, ncap->value + i);
         if (err == NC_ENULLPAD) status = NC_ENULLPAD; /* non-fatal error */
         else if (err != NC_NOERR) { /* error: fail to get the next dim */
             ncmpio_free_NC_dimarray(ncap);
@@ -899,7 +906,10 @@ hdr_get_NC_attr(bufferinfo *gbp, NC_attr **attrpp)
 
     /* get nc_type */
     err = hdr_get_nc_type(gbp, &type);
-    if (err != NC_NOERR) return err;
+    if (err != NC_NOERR) {
+        NCI_Free(name);
+        return err;
+    }
 
     /* get nelems */
     if (gbp->version < 5) {
@@ -912,9 +922,12 @@ hdr_get_NC_attr(bufferinfo *gbp, NC_attr **attrpp)
         err = hdr_get_uint64(gbp, &tmp);
         nelems = (MPI_Offset)tmp;
     }
-    if (err != NC_NOERR) return err;
+    if (err != NC_NOERR) {
+        NCI_Free(name);
+        return err;
+    }
 
-    /* allocate space for attribute object */
+    /* allocate space for attribute object, name will be assigned in attrp */
     err = ncmpio_new_NC_attr(name, type, nelems, &attrp);
     if (err != NC_NOERR) {
         NCI_Free(name);
@@ -1056,17 +1069,29 @@ hdr_get_NC_var(bufferinfo  *gbp,
     if (gbp->version < 5) {
         uint tmp;
         err = hdr_get_uint32(gbp, &tmp);
-        if (err != NC_NOERR) return err;
+        if (err != NC_NOERR) {
+            NCI_Free(name);
+            return err;
+        }
         /* cannot be more than NC_MAX_VAR_DIMS */
-        if (tmp > NC_MAX_VAR_DIMS) DEBUG_RETURN_ERROR(NC_EMAXDIMS)
+        if (tmp > NC_MAX_VAR_DIMS) {
+            NCI_Free(name);
+            DEBUG_RETURN_ERROR(NC_EMAXDIMS)
+        }
         ndims = (int)tmp;
     }
     else {
         uint64 tmp;
         err = hdr_get_uint64(gbp, &tmp);
-        if (err != NC_NOERR) return err;
+        if (err != NC_NOERR) {
+            NCI_Free(name);
+            return err;
+        }
         /* cannot be more than NC_MAX_VAR_DIMS */
-        if (tmp > NC_MAX_VAR_DIMS) DEBUG_RETURN_ERROR(NC_EMAXDIMS)
+        if (tmp > NC_MAX_VAR_DIMS) {
+            NCI_Free(name);
+            DEBUG_RETURN_ERROR(NC_EMAXDIMS)
+        }
         ndims = (int)tmp;
     }
 
