@@ -2,7 +2,7 @@
  *  Copyright (C) 2013, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: ncmpio_subfile.c 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -92,7 +92,6 @@ subfile_create(NC *ncp)
     int myrank, nprocs, color, status=NC_NOERR, mpireturn;
     char path_sf[1024];
     double ratio;
-    MPI_Comm comm_sf;
     MPI_Info info=MPI_INFO_NULL;
 
     MPI_Comm_rank(ncp->comm, &myrank);
@@ -118,9 +117,9 @@ subfile_create(NC *ncp)
 
     /* TODO: fix error when using generated key value.
      * for now, just passing 0 value. */
-    TRACE_COMM(MPI_Comm_split)(ncp->comm, color, myrank, &comm_sf);
+    TRACE_COMM(MPI_Comm_split)(ncp->comm, color, myrank, &ncp->comm_sf);
     if (mpireturn != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(mpireturn, "MPI_Comm_split"); 
+        return ncmpii_error_mpi2nc(mpireturn, "MPI_Comm_split");
 
     sprintf(path_sf, "%s.subfile_%i.%s", ncp->path, color, "nc");
 
@@ -131,14 +130,13 @@ subfile_create(NC *ncp)
 */
 
     void *ncp_sf;
-    status = ncmpio_create(comm_sf, path_sf, ncp->iomode, ncp->ncid, info,
+    status = ncmpio_create(ncp->comm_sf, path_sf, ncp->iomode, ncp->ncid, info,
                            &ncp_sf);
     if (status != NC_NOERR && myrank == 0)
         fprintf(stderr, "%s: error in creating file(%s): %s\n",
                 __func__, path_sf, ncmpi_strerror(status));
 
     ncp->ncp_sf = (NC*) ncp_sf;
-    MPI_Comm_free(&comm_sf);
 /*
     MPI_Info_free(&info);
 */
@@ -152,7 +150,6 @@ ncmpio_subfile_open(NC *ncp)
 {
     int myrank, nprocs, color, status=NC_NOERR, mpireturn;
     char path_sf[1024];
-    MPI_Comm comm_sf;
     double ratio;
 
     MPI_Comm_rank(ncp->comm, &myrank);
@@ -179,9 +176,9 @@ ncmpio_subfile_open(NC *ncp)
 
     /* TODO: fix error when using generated key value.
      * for now, just passing 0 value. */
-    TRACE_COMM(MPI_Comm_split)(ncp->comm, color, myrank, &comm_sf);
+    TRACE_COMM(MPI_Comm_split)(ncp->comm, color, myrank, &ncp->comm_sf);
     if (mpireturn != MPI_SUCCESS)
-        return ncmpii_error_mpi2nc(mpireturn, "MPI_Comm_split"); 
+        return ncmpii_error_mpi2nc(mpireturn, "MPI_Comm_split");
 
     /* char path[1024], file[1024]; */
     /* find_path_and_fname(ncp->path, path, file); */
@@ -189,11 +186,10 @@ ncmpio_subfile_open(NC *ncp)
     sprintf(path_sf, "%s.subfile_%i.%s", ncp->path, color, "nc");
 
     void *ncp_sf;
-    status = ncmpio_open(comm_sf, path_sf, ncp->iomode, ncp->ncid,
+    status = ncmpio_open(ncp->comm_sf, path_sf, ncp->iomode, ncp->ncid,
                          MPI_INFO_NULL, &ncp_sf);
 
     ncp->ncp_sf = (NC*) ncp_sf;
-    MPI_Comm_free(&comm_sf);
     return status;
 }
 
@@ -206,6 +202,7 @@ int ncmpio_subfile_close(NC *ncp)
         status = ncmpio_close(ncp->ncp_sf);
         if (status != NC_NOERR) return status;
         ncp->ncp_sf = NULL;
+        MPI_Comm_free(&ncp->comm_sf);
     }
 
     /* reset values to 0 */
@@ -712,7 +709,7 @@ ncmpio_subfile_getput_vars(NC               *ncp,
     }
 
 #ifdef SUBFILE_DEBUG
-    for (i=0; i<nprocs && myrank==0; i++) {
+    for (i=0; i<nprocs; i++) {
         printf("%d: count_my_req_per_proc[%d]=%d\n", myrank, i, count_my_req_per_proc[i]);
         printf("%d: count_others_req_per_proc[%d]=%d\n", myrank, i, count_others_req_per_proc[i]);
     }
@@ -730,7 +727,7 @@ ncmpio_subfile_getput_vars(NC               *ncp,
 #endif
 
 #ifdef SUBFILE_DEBUG
-    for (i=0; i<nprocs && myrank==0; i++) {
+    for (i=0; i<nprocs; i++) {
         printf("=> %d: count_others_req_per_proc[%d]=%d\n", myrank, i, count_others_req_per_proc[i]);
     }
 #endif
@@ -751,7 +748,7 @@ ncmpio_subfile_getput_vars(NC               *ncp,
      */
     MPI_Datatype ptype;
     int el_size;
-    int isderived, buftype_is_contig;
+    int buftype_is_contig;
     MPI_Offset bnelems;
 
     if (buftype == MPI_DATATYPE_NULL) {
@@ -769,16 +766,16 @@ ncmpio_subfile_getput_vars(NC               *ncp,
         buftype = ncmpii_nc2mpitype(varp->xtype);
     }
 
-    status = ncmpii_dtype_decode(buftype, &ptype, &el_size, &bnelems,
-                                 &isderived, &buftype_is_contig);
+    status = ncmpii_dtype_decode(buftype, &ptype, &el_size, &bnelems, NULL,
+                                 &buftype_is_contig);
     /* bnelems now is the number of ptype in a buftype */
     if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
 
 #ifdef SUBFILE_DEBUG
     MPI_Aint lb, extent;
     MPI_Type_get_extent(buftype, &lb, &extent);
-    printf("rank(%d): var(%s): ptype=0x%x, el_size=%d, bnelems=%d, isderived=%d, buftype_is_contig=%d, lb=%d, extent=%d\n",
-           myrank, varp->name, ptype, el_size, bnelems, isderived, buftype_is_contig, lb, extent);
+    printf("rank(%d): var(%s): ptype=0x%x, el_size=%d, bnelems=%d, buftype_is_contig=%d, lb=%d, extent=%d\n",
+           myrank, varp->name, ptype, el_size, bnelems, buftype_is_contig, lb, extent);
 #endif
 
     /* if buftype is non-contiguous, pack to contiguous buffer*/
@@ -857,8 +854,7 @@ ncmpio_subfile_getput_vars(NC               *ncp,
                                      buf_count_my[myrank],
                                      (!buftype_is_contig?ptype:buftype),
                                      &array_of_requests[nasyncios++],
-                                     reqMode,
-                                     0);
+                                     reqMode);
         if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
     }
 #ifdef SUBFILE_DEBUG
@@ -931,7 +927,7 @@ ncmpio_subfile_getput_vars(NC               *ncp,
 
 #ifdef SUBFILE_DEBUG
     /* DEBUG: print out others_req.{start,count} */
-    for (i=0; i<nprocs && myrank == 0; i++) {
+    for (i=0; i<nprocs; i++) {
         char str_st[100], str_st_org[100], str_ct[100], str_t1[10];
         sprintf(str_st, "%d: others.start(", i);
         sprintf(str_ct, "%d: others.count(", i);
@@ -1045,8 +1041,7 @@ ncmpio_subfile_getput_vars(NC               *ncp,
                                          buf_count_others[i],
                                          (!buftype_is_contig?ptype:buftype),
                                          &array_of_requests[nasyncios++],
-                                         reqMode,
-                                         0);
+                                         reqMode);
             if (status != NC_NOERR) DEBUG_RETURN_ERROR(status)
         }
     }
