@@ -2,7 +2,7 @@
  *  Copyright (C) 2015, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: ncmpio_util.c 3615 2017-12-18 01:12:14Z wkliao $ */
+/* $Id$ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -71,6 +71,23 @@ void ncmpio_set_pnetcdf_hints(NC *ncp, MPI_Info info)
         else if (ncp->chunk < 0) ncp->chunk = 0;
     }
 
+    /* hint on setting in-place byte swap (matters only for Little Endian) */
+    MPI_Info_get(info, "nc_in_place_swap", MPI_MAX_INFO_VAL-1, value, &flag);
+    if (flag) {
+        if (strcasecmp(value, "enable") == 0) {
+            fClr(ncp->flags, NC_MODE_SWAP_OFF);
+            fSet(ncp->flags, NC_MODE_SWAP_ON);
+        }
+        else if (strcasecmp(value, "disable") == 0) {
+            fClr(ncp->flags, NC_MODE_SWAP_ON);
+            fSet(ncp->flags, NC_MODE_SWAP_OFF);
+        }
+        else if (strcasecmp(value, "auto") == 0) {
+            fClr(ncp->flags, NC_MODE_SWAP_ON);
+            fClr(ncp->flags, NC_MODE_SWAP_OFF);
+        }
+    }
+
 #ifdef ENABLE_SUBFILING
     MPI_Info_get(info, "pnetcdf_subfiling", MPI_MAX_INFO_VAL-1, value, &flag);
     if (flag && strcasecmp(value, "enable") == 0)
@@ -87,6 +104,46 @@ void ncmpio_set_pnetcdf_hints(NC *ncp, MPI_Info info)
 #endif
 }
 
+/*----< ncmpio_first_offset() >-----------------------------------------------*/
+/* Returns the file offset of the first variable element accessed by this
+ * request. Note zero-length request should never call this subroutine.
+ */
+int
+ncmpio_first_offset(const NC         *ncp,
+                    const NC_var     *varp,
+                    const MPI_Offset  start[],  /* [varp->ndims] */
+                    MPI_Offset       *offset)   /* OUT: file offset */
+{
+    int i, ndims;
+
+    ndims = varp->ndims; /* number of dimensions of this variable */
+
+    if (ndims == 0) { /* scalar variable */
+        *offset = varp->begin;
+        return NC_NOERR;
+    }
+
+    *offset = 0;
+    if (IS_RECVAR(varp)) {
+        if (ndims > 1) *offset += start[ndims - 1];
+        for (i=1; i<ndims-1; i++)
+            *offset += start[i] * varp->dsizes[i+1];
+        *offset *= varp->xsz;  /* multiply element size */
+        *offset += start[0] * ncp->recsize;
+    }
+    else {
+        if (ndims > 1) *offset += start[0] * varp->dsizes[1];
+        for (i=1; i<ndims-1; i++)
+            *offset += start[i] * varp->dsizes[i+1];
+        *offset += start[ndims-1];
+        *offset *= varp->xsz;  /* multiply element size */
+    }
+
+    *offset += varp->begin; /* beginning file offset of this variable */
+
+    return NC_NOERR;
+}
+
 /*----< ncmpio_last_offset() >-----------------------------------------------*/
 /* Returns the file offset of the last variable element accessed by this
  * request.
@@ -96,23 +153,23 @@ void ncmpio_set_pnetcdf_hints(NC *ncp, MPI_Info info)
 int
 ncmpio_last_offset(const NC         *ncp,
                    const NC_var     *varp,
-                   const MPI_Offset  start[],   /* [varp->ndims] */
-                   const MPI_Offset  count[],   /* [varp->ndims] */
-                   const MPI_Offset  stride[],  /* [varp->ndims] */
-                   const int         reqMode,
+                   const MPI_Offset  start[],    /* [varp->ndims] */
+                   const MPI_Offset  count[],    /* [varp->ndims] */
+                   const MPI_Offset  stride[],   /* [varp->ndims] */
                    MPI_Offset       *offset_ptr) /* OUT: file offset */
 {
+    int i, ndims;
     MPI_Offset offset, *last_indx=NULL;
-    int i, ndims, firstDim = 0;
 
     offset = varp->begin; /* beginning file offset of this variable */
     ndims  = varp->ndims; /* number of dimensions of this variable */
 
-    if (ndims == 0) {
-        *offset_ptr = varp->begin + varp->xsz;
+    if (ndims == 0) { /* scalar variable */
+        *offset_ptr = varp->begin;
         return NC_NOERR;
     }
 
+    /* when count == NULL, this is called from a var API */
     if (count != NULL) {
         last_indx = (MPI_Offset*) NCI_Malloc((size_t)ndims * SIZEOF_MPI_OFFSET);
 
@@ -133,30 +190,8 @@ ncmpio_last_offset(const NC         *ncp,
         last_indx = (MPI_Offset*) start;
     }
 
-    /* check whether last_indx is valid */
-
-    firstDim = 0;
-    /* check NC_EINVALCOORDS for record dimension */
-    if (varp->shape[0] == NC_UNLIMITED) {
-        if (ncp->format < 5 && last_indx[0] > NC_MAX_UINT) { /* CDF-1 and 2 */
-            if (count != NULL) NCI_Free(last_indx);
-            DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
-        }
-        /* for record variable, [0] is the NC_UNLIMITED dimension */
-        if (fIsSet(reqMode, NC_REQ_RD) && last_indx[0] >= ncp->numrecs) {
-            /* read cannot go beyond current numrecs */
-            if (count != NULL) NCI_Free(last_indx);
-            DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
-        }
-        firstDim = 1; /* done for checking the record dimension */
-    }
-    /* continue to check NC_EINVALCOORDS for the rest dimensions */
-    for (i=firstDim; i<ndims; i++) {
-        if (last_indx[i] < 0 || last_indx[i] >= varp->shape[i]) {
-            if (count != NULL) NCI_Free(last_indx);
-            DEBUG_RETURN_ERROR(NC_EINVALCOORDS)
-        }
-    }
+    /* Note check NC_EINVALCOORDS and NC_EEDGE already done in
+     * dispatchers/var_getput.m4 */
 
     if (varp->shape[0] == NC_UNLIMITED)
         offset += last_indx[0] * ncp->recsize;
@@ -245,8 +280,9 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
                  MPI_Offset    bufcount,
                  MPI_Datatype  buftype,
                  int           buftype_is_contig,
-                 MPI_Offset    nelems, /* no. elements in etype in buf */
-                 MPI_Datatype  etype,  /* element type in buftype */
+                 MPI_Offset    nelems, /* no. elements in buf */
+                 MPI_Datatype  itype,  /* element type in buftype */
+                 int           el_size,/* size of itype */
                  MPI_Datatype  imaptype,
                  int           need_convert,
                  int           need_swap,
@@ -254,12 +290,11 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
                  void         *buf,    /* user buffer */
                  void         *xbuf)   /* already allocated, in external type */
 {
-    int err=NC_NOERR, el_size, position;
-    void *lbuf=NULL, *cbuf=NULL; 
+    int err=NC_NOERR, position, free_lbuf=0, free_cbuf=0;
+    void *lbuf=NULL, *cbuf=NULL;
     MPI_Offset ibuf_size;
 
     /* check byte size of buf (internal representation) */
-    MPI_Type_size(etype, &el_size);
     ibuf_size = nelems * el_size;
     if (ibuf_size > INT_MAX) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
 
@@ -275,13 +310,19 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
              * constructing xbuf */
             lbuf = NCI_Malloc((size_t)ibuf_size);
             if (lbuf == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
+            free_lbuf = 1;
         }
 
-        /* pack buf into lbuf based on buftype */
-        if (bufcount > INT_MAX) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
-        position = 0;
-        MPI_Pack(buf, (int)bufcount, buftype, lbuf, (int)ibuf_size,
-                 &position, MPI_COMM_SELF);
+        if (buf != lbuf) {
+            /* pack buf into lbuf based on buftype */
+            if (bufcount > INT_MAX) {
+                if (free_lbuf) NCI_Free(lbuf);
+                DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
+            }
+            position = 0;
+            MPI_Pack(buf, (int)bufcount, buftype, lbuf, (int)ibuf_size,
+                     &position, MPI_COMM_SELF);
+        }
     }
     else /* for contiguous case, we reuse buf */
         lbuf = buf;
@@ -295,7 +336,11 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
             /* in this case, allocate cbuf and cbuf will be freed before
              * constructing xbuf */
             cbuf = NCI_Malloc((size_t)ibuf_size);
-            if (cbuf == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
+            if (cbuf == NULL) {
+                if (free_lbuf) NCI_Free(lbuf);
+                DEBUG_RETURN_ERROR(NC_ENOMEM)
+            }
+            free_cbuf = 1;
         }
 
         /* pack lbuf to cbuf based on imaptype */
@@ -305,7 +350,10 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
         MPI_Type_free(&imaptype);
 
         /* lbuf is no longer needed */
-        if (lbuf != buf) NCI_Free(lbuf);
+        if (free_lbuf) {
+            NCI_Free(lbuf);
+            free_lbuf = 0;
+        }
     }
     else /* not a true varm call: reuse lbuf */
         cbuf = lbuf;
@@ -315,43 +363,43 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
      */
     if (need_convert) {
         /* user buf type does not match nc var type defined in file */
-        void *fillp; /* fill value in internal representation */
+        char  tmpbuf[8];
+        void *fillp=tmpbuf; /* fill value in internal representation */
 
         /* find the fill value */
-        fillp = NCI_Malloc((size_t)varp->xsz);
         ncmpio_inq_var_fill(varp, fillp);
 
         /* datatype conversion + byte-swap from cbuf to xbuf */
         switch(varp->xtype) {
             case NC_BYTE:
-                err = ncmpii_putn_NC_BYTE(fmt,xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_BYTE(fmt,xbuf,cbuf,nelems,itype,fillp);
                 break;
             case NC_UBYTE:
-                err = ncmpii_putn_NC_UBYTE(xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_UBYTE(xbuf,cbuf,nelems,itype,fillp);
                 break;
             case NC_SHORT:
-                err = ncmpii_putn_NC_SHORT(xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_SHORT(xbuf,cbuf,nelems,itype,fillp);
                 break;
             case NC_USHORT:
-                err = ncmpii_putn_NC_USHORT(xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_USHORT(xbuf,cbuf,nelems,itype,fillp);
                 break;
             case NC_INT:
-                err = ncmpii_putn_NC_INT(xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_INT(xbuf,cbuf,nelems,itype,fillp);
                 break;
             case NC_UINT:
-                err = ncmpii_putn_NC_UINT(xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_UINT(xbuf,cbuf,nelems,itype,fillp);
                 break;
             case NC_FLOAT:
-                err = ncmpii_putn_NC_FLOAT(xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_FLOAT(xbuf,cbuf,nelems,itype,fillp);
                 break;
             case NC_DOUBLE:
-                err = ncmpii_putn_NC_DOUBLE(xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_DOUBLE(xbuf,cbuf,nelems,itype,fillp);
                 break;
             case NC_INT64:
-                err = ncmpii_putn_NC_INT64(xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_INT64(xbuf,cbuf,nelems,itype,fillp);
                 break;
             case NC_UINT64:
-                err = ncmpii_putn_NC_UINT64(xbuf,cbuf,nelems,etype,fillp);
+                err = ncmpii_putn_NC_UINT64(xbuf,cbuf,nelems,itype,fillp);
                 break;
             default:
                 err = NC_EBADTYPE; /* this never happens */
@@ -365,9 +413,8 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
 	 * the external data type, it is not considered a fatal error. This
 	 * request must continue to finish.
          */
-
-        NCI_Free(fillp);
-        if (cbuf != buf) NCI_Free(cbuf);
+        if (free_cbuf) NCI_Free(cbuf);
+        if (free_lbuf) NCI_Free(lbuf);
     }
     else {
         if (cbuf == buf && xbuf != buf)
@@ -376,6 +423,7 @@ ncmpio_pack_xbuf(int           fmt,    /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
         if (need_swap) /* perform array in-place byte swap on xbuf */
             ncmpii_in_swapn(xbuf, nelems, varp->xsz);
     }
+
     return err;
 }
 
@@ -416,20 +464,20 @@ ncmpio_unpack_xbuf(int           fmt,   /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
                    MPI_Offset    bufcount,
                    MPI_Datatype  buftype,
                    int           buftype_is_contig,
-                   MPI_Offset    nelems, /* no. elements in etype in buf */
-                   MPI_Datatype  etype,  /* element type in buftype */
+                   MPI_Offset    nelems, /* no. elements in buf */
+                   MPI_Datatype  itype,  /* element type in buftype */
                    MPI_Datatype  imaptype,
                    int           need_convert,
                    int           need_swap,
                    void         *buf,  /* user buffer */
                    void         *xbuf) /* already allocated, in external type */
 {
-    int err=NC_NOERR, el_size, position;
-    void *lbuf=NULL, *cbuf=NULL; 
+    int err=NC_NOERR, el_size, position, free_lbuf=0, free_cbuf=0;
+    void *lbuf=NULL, *cbuf=NULL;
     MPI_Offset ibuf_size;
 
     /* check byte size of buf (internal representation) */
-    MPI_Type_size(etype, &el_size);
+    MPI_Type_size(itype, &el_size);
     ibuf_size = nelems * el_size;
     if (ibuf_size > INT_MAX) DEBUG_RETURN_ERROR(NC_EINTOVERFLOW)
 
@@ -445,39 +493,40 @@ ncmpio_unpack_xbuf(int           fmt,   /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
         else {
             cbuf = NCI_Malloc(ibuf_size);
             if (cbuf == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
+            free_cbuf = 1;
         }
 
         /* datatype conversion + byte-swap from xbuf to cbuf */
         switch(varp->xtype) {
             case NC_BYTE:
-                err = ncmpii_getn_NC_BYTE(fmt,xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_BYTE(fmt,xbuf,cbuf,nelems,itype);
                 break;
             case NC_UBYTE:
-                err = ncmpii_getn_NC_UBYTE(xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_UBYTE(xbuf,cbuf,nelems,itype);
                 break;
             case NC_SHORT:
-                err = ncmpii_getn_NC_SHORT(xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_SHORT(xbuf,cbuf,nelems,itype);
                 break;
             case NC_USHORT:
-                err = ncmpii_getn_NC_USHORT(xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_USHORT(xbuf,cbuf,nelems,itype);
                 break;
             case NC_INT:
-                err = ncmpii_getn_NC_INT(xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_INT(xbuf,cbuf,nelems,itype);
                 break;
             case NC_UINT:
-                err = ncmpii_getn_NC_UINT(xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_UINT(xbuf,cbuf,nelems,itype);
                 break;
             case NC_FLOAT:
-                err = ncmpii_getn_NC_FLOAT(xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_FLOAT(xbuf,cbuf,nelems,itype);
                 break;
             case NC_DOUBLE:
-                err = ncmpii_getn_NC_DOUBLE(xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_DOUBLE(xbuf,cbuf,nelems,itype);
                 break;
             case NC_INT64:
-                err = ncmpii_getn_NC_INT64(xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_INT64(xbuf,cbuf,nelems,itype);
                 break;
             case NC_UINT64:
-                err = ncmpii_getn_NC_UINT64(xbuf,cbuf,nelems,etype);
+                err = ncmpii_getn_NC_UINT64(xbuf,cbuf,nelems,itype);
                 break;
             default:
                 err = NC_EBADTYPE; /* this never happens */
@@ -507,7 +556,11 @@ ncmpio_unpack_xbuf(int           fmt,   /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
          * In this case, cbuf cannot be buf and lbuf cannot be buf.
          */
         lbuf = NCI_Malloc((size_t)ibuf_size);
-        if (lbuf == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
+        if (lbuf == NULL) {
+            if (free_cbuf) NCI_Free(cbuf);
+            DEBUG_RETURN_ERROR(NC_ENOMEM)
+        }
+        free_lbuf = 1;
     }
     else if (imaptype == MPI_DATATYPE_NULL) /* not varm */
         lbuf = cbuf;
@@ -521,12 +574,11 @@ ncmpio_unpack_xbuf(int           fmt,   /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
         MPI_Unpack(cbuf, (int)ibuf_size, &position, lbuf, 1, imaptype,
                    MPI_COMM_SELF);
         MPI_Type_free(&imaptype);
-        /* done with cbuf */
-        if (cbuf != xbuf) NCI_Free(cbuf);
     }
 
     /* unpacked lbuf into buf based on buftype -----------------------------*/
-    if (!buftype_is_contig) {
+    if (!buftype_is_contig && lbuf != buf) {
+        /* no need unpack when buftype is used in MPI_File_read (lbuf == buf) */
         if (bufcount > INT_MAX) {
             if (err == NC_NOERR)
                 DEBUG_ASSIGN_ERROR(err, NC_EINTOVERFLOW)
@@ -535,10 +587,10 @@ ncmpio_unpack_xbuf(int           fmt,   /* NC_FORMAT_CDF2 NC_FORMAT_CDF5 etc. */
             position = 0;
             MPI_Unpack(lbuf, (int)ibuf_size, &position, buf, (int)bufcount,
                        buftype, MPI_COMM_SELF);
-            /* done with lbuf */
-            if (lbuf != buf && lbuf != xbuf) NCI_Free(lbuf);
         }
     }
+    if (free_cbuf) NCI_Free(cbuf);
+    if (free_lbuf) NCI_Free(lbuf);
 
     return err;
 }
