@@ -2,7 +2,7 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: ncmpio_var.c 3440 2017-10-15 18:46:02Z wkliao $ */
+/* $Id$ */
 
 /*
  * This file implements the corresponding APIs defined in
@@ -70,11 +70,6 @@ ncmpio_new_NC_var(char *name, int ndims)
     varp->name     = name;         /* name has been malloc-ed */
     varp->name_len = strlen(name); /* name has been NULL checked */
     varp->ndims    = ndims;
-#if 0
-    varp->xsz      = 0;
-    varp->len      = 0;
-    varp->begin    = 0;
-#endif
 
     return varp;
 }
@@ -130,14 +125,17 @@ ncmpio_free_NC_vararray(NC_vararray *ncap)
     assert(ncap != NULL);
     if (ncap->ndefined == 0) return;
 
-    assert(ncap->value != NULL);
-    for (i=0; i<ncap->ndefined; i++) {
-        if (ncap->value[i] != NULL)
-            ncmpio_free_NC_var(ncap->value[i]);
+    if (ncap->value != NULL) {
+        /* when error is detected reading NC_VARIABLE tag, ncap->ndefined can
+         * be > 0 and ncap->value is still NULL
+         */
+        for (i=0; i<ncap->ndefined; i++) {
+            if (ncap->value[i] != NULL)
+                ncmpio_free_NC_var(ncap->value[i]);
+        }
+        NCI_Free(ncap->value);
+        ncap->value    = NULL;
     }
-
-    NCI_Free(ncap->value);
-    ncap->value    = NULL;
     ncap->ndefined = 0;
 
 #ifndef SEARCH_NAME_LINEARLY
@@ -192,22 +190,6 @@ ncmpio_dup_NC_vararray(NC_vararray       *ncap,
     return NC_NOERR;
 }
 
-#if 0
-/*----< elem_NC_vararray() >-------------------------------------------------*/
-inline static NC_var *
-elem_NC_vararray(const NC_vararray *ncap,
-                 int                varid)
-{
-    assert(ncap != NULL);
-    /* cast needed for braindead systems with signed MPI_Offset */
-    if ((varid < 0) ||  ncap->ndefined == 0 || varid >= ncap->ndefined)
-        return NULL;
-
-    assert(ncap->value != NULL);
-
-    return ncap->value[varid];
-}
-#endif
 
 /* End vararray per se */
 
@@ -303,6 +285,9 @@ ncmpio_NC_var_shape64(NC_var            *varp,
 
     /*
      * compute dsizes[], from right to left product of shape
+     * For example, a 3D array of size 5x4x3 in C order,
+     * For fixed-size variable: dsizes[0]=60 dsizes[1]=12 dsizes[2]=3
+     * For record     variable: dsizes[0]=12 dsizes[1]=12 dsizes[2]=3
      */
     product = 1;
     if (varp->ndims == 1) {
@@ -350,25 +335,6 @@ out :
     return NC_NOERR;
 }
 
-#if 0
-/*----< ncmpio_NC_lookupvar() >----------------------------------------------*/
-/* Given valid ncp and varid, return varp */
-int
-ncmpio_NC_lookupvar(NC      *ncp,
-                    int      varid,
-                    NC_var **varp)
-{
-    if (varid == NC_GLOBAL) /* Global is error in this context */
-        DEBUG_RETURN_ERROR(NC_EGLOBAL)
-
-    *varp = elem_NC_vararray(&ncp->vars, varid);
-    if (*varp == NULL) /* could not find variable with varid */
-        DEBUG_RETURN_ERROR(NC_ENOTVAR)
-
-    return NC_NOERR;
-}
-#endif
-
 /*----< ncmpio_def_var() >---------------------------------------------------*/
 int
 ncmpio_def_var(void       *ncdp,
@@ -523,12 +489,7 @@ ncmpio_inq_var(void       *ncdp,
         return NC_NOERR;
     }
 
-#if 0
-    varp = elem_NC_vararray(&ncp->vars, varid);
-    if (varp == NULL) DEBUG_RETURN_ERROR(NC_ENOTVAR)
-#else
     varp = ncp->vars.value[varid];
-#endif
 
     if (name != NULL)
         /* in PnetCDF, name is always NULL character terminated */
@@ -590,7 +551,7 @@ ncmpio_rename_var(void       *ncdp,
     /* sanity check for ncdp and varid has been done in dispatchers */
     varp = ncp->vars.value[varid];
 
-    /* create a normalized character string */ 
+    /* create a normalized character string */
     err = ncmpii_utf8_normalize(newname, &nnewname);
     if (err != NC_NOERR) goto err_check;
 
