@@ -2,7 +2,7 @@
  *  Copyright (C) 2015, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: ncoffsets.c 3550 2017-11-22 03:52:30Z wkliao $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,14 +25,14 @@ static int verbose_debug;
 
 #define DEBUG_RETURN_ERROR(err) {                             \
     if (verbose_debug)                                        \
-        fprintf(stderr, "Error code %s at line %d of %s\n",   \
-        ncmpii_err_code_name(err),__LINE__,__FILE__);         \
+        fprintf(stderr, "Error at line %d of %s: %s\n",       \
+        __LINE__,__FILE__,#err);                              \
     return err;                                               \
 }
 #define DEBUG_ASSIGN_ERROR(status, err) {                     \
     if (verbose_debug)                                        \
-        fprintf(stderr, "Error code %s at line %d of %s\n",   \
-        ncmpii_err_code_name(err),__LINE__,__FILE__);         \
+        fprintf(stderr, "Error at line %d of %s: %s\n",       \
+        __LINE__,__FILE__,#err);                              \
     status = err;                                             \
 }
 
@@ -51,6 +51,7 @@ static int verbose_debug;
 #define NC_EBADDIM	(-46)	/**< Invalid dimension id or name */
 #define NC_EUNLIMPOS	(-47)	/**< NC_UNLIMITED in the wrong index */
 #define NC_ENOTNC	(-51)	/**< Not a netcdf file (file format violates CDF specification) */
+#define NC_EUNLIMIT    	(-54)	   /**< NC_UNLIMITED size already in use */
 #define NC_EVARSIZE     (-62)   /**< One or more variable sizes violate format constraints */
 
 #define NC_UNLIMITED 0L
@@ -126,8 +127,9 @@ typedef struct {
 } NC_dim;
 
 typedef struct NC_dimarray {
-    int      nalloc;    /* number allocated >= ndefined */
-    int      ndefined;  /* number of defined dimensions */
+    int      nalloc;       /* number allocated >= ndefined */
+    int      ndefined;     /* number of defined dimensions */
+    int      unlimited_id; /* ID of unlimited dimension */
     NC_dim **value;
 } NC_dimarray;
 
@@ -193,10 +195,9 @@ typedef struct bufferinfo {
 
 /*
  * "magic number" at beginning of file: 0x43444601 (big endian)
+ * "CDF1", "CDF2", or "CDF5"
  */
-static const char ncmagic1[] = {'C', 'D', 'F', 0x01};
-static const char ncmagic2[] = {'C', 'D', 'F', 0x02};
-static const char ncmagic5[] = {'C', 'D', 'F', 0x05};
+#define NC_MAGIC_LEN 4
 
 const char * ncmpii_err_code_name(int err);
 
@@ -411,11 +412,11 @@ ncmpii_NC_var_shape64(NC                *ncp,
 out :
     /*
      * For CDF-1 and CDF-2 formats, the total number of array elements
-     * cannot exceed 2^32, unless this variable is the last fixed-size 
-     * variable, there is no record variable, and the file starting 
-     * offset of this variable is less than 2GiB. 
-     * We will check this in ncmpi_enddef() which calls ncmpii_NC_enddef() 
-     * which calls ncmpii_NC_check_vlens() 
+     * cannot exceed 2^32, unless this variable is the last fixed-size
+     * variable, there is no record variable, and the file starting
+     * offset of this variable is less than 2GiB.
+     * We will check this in ncmpi_enddef() which calls ncmpii_NC_enddef()
+     * which calls ncmpii_NC_check_vlens()
     if (ncp->flags != 5 && product >= X_UINT_MAX)
         DEBUG_RETURN_ERROR(NC_EVARSIZE);
      */
@@ -765,7 +766,7 @@ ncmpii_hdr_len_NC(const NC *ncp)
         sizeof_off_t = X_SIZEOF_INT; /* 4-byte integer in CDF-1 */
     }
 
-    xlen  = sizeof(ncmagic1);                                          /* magic */
+    xlen  = NC_MAGIC_LEN;                                              /* magic */
     xlen += sizeof_t;                                                  /* numrecs */
     xlen += hdr_len_NC_dimarray(&ncp->dims,   sizeof_t);               /* dim_list */
     xlen += hdr_len_NC_attrarray(&ncp->attrs, sizeof_t);               /* gatt_list */
@@ -792,7 +793,7 @@ hdr_fetch(bufferinfo *gbp) {
     gbp->pos = gbp->base;
 
     lseek(gbp->fd, (gbp->offset)-slack, SEEK_SET);
-    size_t read_amount = read(gbp->fd, gbp->base, gbp->size);
+    ssize_t read_amount = read(gbp->fd, gbp->base, gbp->size);
     if (read_amount == -1) {
         fprintf(stderr,"ERROR at line %d: read error %s\n",__LINE__,strerror(errno));
         exit(1);
@@ -826,7 +827,7 @@ hdr_get_NCtype(bufferinfo *gbp,
     if (status != NC_NOERR) return status;
 
     /* get a 4-byte integer */
-    *typep = get_uint32(gbp);
+    *typep = (NCtype) get_uint32(gbp);
 
     return NC_NOERR;
 }
@@ -881,10 +882,10 @@ hdr_get_NC_name(bufferinfo  *gbp,
     int status;
     size_t  nchars, nbytes, padding, bufremain, strcount;
     NC_string *ncstrp;
-    char *cpos, pad[X_ALIGN-1];
+    char *cpos;
 
     /* get nelems */
-    if (gbp->version == 5) 
+    if (gbp->version == 5)
         nchars = get_uint64(gbp);
     else
         nchars = get_uint32(gbp);
@@ -941,6 +942,7 @@ hdr_get_NC_name(bufferinfo  *gbp,
          * and discussion in NetCDF Github issue
          * https://github.com/Unidata/netcdf-c/issues/657.
          */
+        char pad[X_ALIGN-1];
         memset(pad, 0, X_ALIGN-1);
         if (memcmp(gbp->pos, pad, padding) != 0) {
             free(ncstrp);
@@ -972,6 +974,7 @@ ncmpii_new_x_NC_dim(NC_string *name)
 /*----< hdr_get_NC_dim() >----------------------------------------------------*/
 static int
 hdr_get_NC_dim(bufferinfo  *gbp,
+               int          unlimited_id,
                NC_dim     **dimpp)
 {
     /* netCDF file format:
@@ -982,6 +985,7 @@ hdr_get_NC_dim(bufferinfo  *gbp,
      *              <non-negative INT64>  // CDF-5
      */
     int status;
+    long long dim_length;
     NC_string *ncstrp;
     NC_dim *dimp;
 
@@ -989,13 +993,20 @@ hdr_get_NC_dim(bufferinfo  *gbp,
     status = hdr_get_NC_name(gbp, &ncstrp);
     if (status != NC_NOERR) return status;
 
-    dimp = ncmpii_new_x_NC_dim(ncstrp);
-
     /* get dim_length */
-    if (gbp->version == 5) 
-        dimp->size = get_uint64(gbp);
+    if (gbp->version == 5)
+        dim_length = get_uint64(gbp);
     else
-        dimp->size = get_uint32(gbp);
+        dim_length = get_uint32(gbp);
+
+    /* check if unlimited_id already set */
+    if (unlimited_id != -1 && dim_length == 0) {
+        free(ncstrp);
+        DEBUG_RETURN_ERROR(NC_EUNLIMIT);
+    }
+
+    dimp = ncmpii_new_x_NC_dim(ncstrp);
+    dimp->size = dim_length;
 
     *dimpp = dimp;
     return NC_NOERR;
@@ -1054,12 +1065,14 @@ hdr_get_NC_dimarray(bufferinfo  *gbp,
     if (status != NC_NOERR) return status;
 
     /* get nelems */
-    if (gbp->version == 5) 
+    if (gbp->version == 5)
         ndefined = get_uint64(gbp);
     else
         ndefined = get_uint32(gbp);
     ncap->ndefined = (int)ndefined;
 
+    ncap->unlimited_id = -1;
+
     if (ndefined == 0) {
         if (type != NC_UNSPECIFIED)
             DEBUG_RETURN_ERROR(NC_ENOTNC);
@@ -1071,12 +1084,14 @@ hdr_get_NC_dimarray(bufferinfo  *gbp,
         ncap->nalloc = (int)ndefined;
 
         for (i=0; i<ndefined; i++) {
-            status = hdr_get_NC_dim(gbp, ncap->value + i);
+            status = hdr_get_NC_dim(gbp, ncap->unlimited_id, ncap->value + i);
             if (status != NC_NOERR) { /* error: fail to get the next dim */
                 ncap->ndefined = i;
                 ncmpii_free_NC_dimarray(ncap);
                 return status;
             }
+            if (ncap->value[i]->size == NC_UNLIMITED)
+                ncap->unlimited_id = i; /* ID of unlimited dimension */
         }
     }
 
@@ -1103,7 +1118,6 @@ hdr_get_NC_attrV(bufferinfo *gbp,
      */
     int status;
     void *value = attrp->xvalue;
-    char pad[X_ALIGN-1];
     size_t nbytes, padding, bufremain, attcount;
 
     nbytes = attrp->nelems * ncmpix_len_nctype(attrp->type);
@@ -1151,6 +1165,7 @@ hdr_get_NC_attrV(bufferinfo *gbp,
          * and discussion in NetCDF Github issue
          * https://github.com/Unidata/netcdf-c/issues/657.
          */
+        char pad[X_ALIGN-1];
         memset(pad, 0, X_ALIGN-1);
         if (memcmp(gbp->pos, pad, (size_t)padding) != 0)
             DEBUG_RETURN_ERROR(NC_ENOTNC);
@@ -1251,7 +1266,7 @@ hdr_get_NC_attr(bufferinfo  *gbp,
     }
 
     /* get nelems */
-    if (gbp->version == 5) 
+    if (gbp->version == 5)
         nelems = get_uint64(gbp);
     else
         nelems = get_uint32(gbp);
@@ -1316,7 +1331,7 @@ hdr_get_NC_attrarray(bufferinfo   *gbp,
     if (status != NC_NOERR) return status;
 
     /* get nelems */
-    if (gbp->version == 5) 
+    if (gbp->version == 5)
         ndefined = get_uint64(gbp);
     else
         ndefined = get_uint32(gbp);
@@ -1418,7 +1433,7 @@ hdr_get_NC_var(bufferinfo  *gbp,
     if (status != NC_NOERR) return status;
 
     /* nelems */
-    if (gbp->version == 5) 
+    if (gbp->version == 5)
         ndims = get_uint64(gbp);
     else
         ndims = get_uint32(gbp);
@@ -1433,7 +1448,7 @@ hdr_get_NC_var(bufferinfo  *gbp,
             ncmpii_free_NC_var(varp);
             return status;
         }
-        if (gbp->version == 5) 
+        if (gbp->version == 5)
             varp->dimids[dim] = get_uint64(gbp);
         else
             varp->dimids[dim] = get_uint32(gbp);
@@ -1454,7 +1469,7 @@ hdr_get_NC_var(bufferinfo  *gbp,
     }
 
     /* get vsize */
-    if (gbp->version == 5) 
+    if (gbp->version == 5)
         varp->len = get_uint64(gbp);
     else
         varp->len = get_uint32(gbp);
@@ -1528,7 +1543,7 @@ hdr_get_NC_vararray(bufferinfo  *gbp,
     if (status != NC_NOERR) return status;
 
     /* get nelems (number of variables) from gbp buffer */
-    if (gbp->version == 5) 
+    if (gbp->version == 5)
         ndefined = get_uint64(gbp);
     else
         ndefined = get_uint32(gbp);
@@ -1601,19 +1616,19 @@ ncmpii_hdr_get_NC(int fd, NC *ncp)
     /* don't need to worry about CDF-1 or CDF-2
      * if the first bits are not 'CDF'
      */
-    if (memcmp(magic, ncmagic1, sizeof(ncmagic1)-1) != 0) {
+    if (strncmp(magic, "CDF", NC_MAGIC_LEN-1) != 0) {
         DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
         goto fn_exit;
     }
 
     /* check version number in last byte of magic */
-    if (magic[sizeof(ncmagic1)-1] == 0x1) {
+    if (magic[NC_MAGIC_LEN-1] == 0x1) {
         getbuf.version = 1;
         ncp->flags = 1;
-    } else if (magic[sizeof(ncmagic1)-1] == 0x2) {
+    } else if (magic[NC_MAGIC_LEN-1] == 0x2) {
         getbuf.version = 2;
         ncp->flags = 2;
-    } else if (magic[sizeof(ncmagic1)-1] == 0x5) {
+    } else if (magic[NC_MAGIC_LEN-1] == 0x5) {
         getbuf.version = 5;
         ncp->flags = 5;
     } else {
@@ -1626,7 +1641,7 @@ ncmpii_hdr_get_NC(int fd, NC *ncp)
     if(status != NC_NOERR) goto fn_exit;
 
     /* get numrecs from getbuf into ncp */
-    if (getbuf.version == 5) 
+    if (getbuf.version == 5)
         ncp->numrecs = get_uint64(&getbuf);
     else
         ncp->numrecs = get_uint32(&getbuf);
@@ -1708,6 +1723,7 @@ ncmpii_err_code_name(int err)
       case NC_EBADDIM:   return "NC_EBADDIM";
       case NC_EUNLIMPOS: return "NC_EUNLIMPOS";
       case NC_ENOTNC:    return "NC_ENOTNC";
+      case NC_EUNLIMIT:  return "NC_EUNLIMIT";
       case NC_EVARSIZE : return "NC_EVARSIZE";
       default:
          printf("Unknown error code %d\n",err);
@@ -1814,7 +1830,7 @@ usage(char *cmd)
 "       [-x]            Check gaps in fixed-size variables, output 1 if gaps\n"
 "                       are found, 0 for otherwise.\n"
 "       file            Input netCDF file name\n"
-"*Parallel netCDF library version 1.9.0 of 19 Dec 2017\n";
+"*PnetCDF library version PNETCDF_RELEASE_VERSION of PNETCDF_RELEASE_DATE\n";
     fprintf(stderr, help, cmd);
 }
 
@@ -1822,15 +1838,13 @@ usage(char *cmd)
 int main(int argc, char *argv[])
 {
     extern int optind;
-    char *filename, *cmd, *env_str;
+    char *filename, *env_str;
     int i, j, err, opt;
     int print_var_size=0, print_gap=0, check_gap=0, print_all_rec=0;
     NC *ncp;
     struct fspec *fspecp=NULL;
 
     fspecp = (struct fspec*) calloc(1, sizeof(struct fspec));
-    cmd = (char*) malloc(strlen(argv[0])+1);
-    strcpy(cmd,argv[0]);
 
     /* get command-line arguments */
     while ((opt = getopt(argc, argv, "v:sghqxr")) != EOF) {
@@ -1846,26 +1860,22 @@ int main(int argc, char *argv[])
             case 'x': check_gap = 1;
                       break;
             case 'h':
-            default:  usage(cmd);
+            default:  usage(argv[0]);
                       free(fspecp);
                       return 0;
         }
     }
-    argc -= optind;
-    argv += optind;
-    if (argc != 1) {
-        fprintf(stderr, "%s: missing file name\n", cmd);
-        usage(cmd);
+    if (argv[optind] == NULL) { /* input file name is mandatory */
+        fprintf(stderr, "%s: missing file name\n", argv[0]);
+        usage(argv[0]);
         if (fspecp->varp != NULL) free(fspecp->varp);
         for (i=0; i<fspecp->nlvars; i++)
             free(fspecp->lvars[i]);
         if (fspecp->lvars != NULL) free(fspecp->lvars);
         free(fspecp);
-        free(cmd);
         return 1;
     }
-    free(cmd);
-    filename = argv[0]; /* required argument */
+    filename = argv[optind]; /* required argument */
 
     verbose_debug = 0;
     env_str = getenv("PNETCDF_VERBOSE_DEBUG_MODE");
@@ -1888,7 +1898,8 @@ int main(int argc, char *argv[])
     /* read the header from file */
     err = ncmpii_hdr_get_NC(fd, ncp);
     if (err != NC_NOERR) {
-        fprintf(stderr,"Error: %s\n", ncmpii_err_code_name(err));
+        fprintf(stderr,"Error when reading header of file \"%s\": %s\n",
+                filename, ncmpii_err_code_name(err));
         exit(1);
     }
 
@@ -1930,7 +1941,7 @@ int main(int argc, char *argv[])
     printf("file header:\n");
     printf("\tsize   = %lld bytes\n",ncp->xsz);
     printf("\textent = %lld bytes\n",ncp->begin_var);
-    
+
     /* print dimensions */
     if (ncp->dims.ndefined > 0) printf("\ndimensions:\n");
     for (i=0; i<ncp->dims.ndefined; i++) {
