@@ -2,7 +2,7 @@
  *  Copyright (C) 2003, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: ncvalidator.c 3605 2017-12-16 09:21:12Z wkliao $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,7 +31,7 @@
 #endif
 #endif
 
-static int verbose;
+static int verbose, trace;
 static int repair;
 static const char nada[4] = {0, 0, 0, 0};
 
@@ -108,7 +108,8 @@ typedef struct {
 } NC_dim;
 
 typedef struct NC_dimarray {
-    int      ndefined;  /* number of defined dimensions */
+    int      ndefined;     /* number of defined dimensions */
+    int      unlimited_id; /* ID of unlimited dimension */
     NC_dim **value;
 } NC_dimarray;
 
@@ -181,15 +182,17 @@ typedef struct bufferinfo {
 #define NC_EUNLIMPOS    (-47)   /**< NC_UNLIMITED in the wrong index */
 #define NC_EMAXVARS     (-48)
 #define NC_ENOTNC       (-51)   /**< Not a netcdf file (file format violates CDF specification) */
+#define NC_EUNLIMIT     (-54)   /**< NC_UNLIMITED size already in use */
 #define NC_ENOMEM       (-61)   /**< Memory allocation (malloc) failure */
 #define NC_EVARSIZE     (-62)   /**< One or more variable sizes violate format constraints */
 #define NC_EFILE        (-204)  /**< Unknown error in file operation */
+#define NC_ENOTSUPPORT  (-214)  /**< Feature is not yet supported */
 #define NC_ENULLPAD     (-134)  /**< Header Bytes not Null-Byte padded */
 
 /*
- * "magic number" at beginning of file: 0x43444601 (big Endian) 
+ * "magic number" at beginning of file: 0x43444601 (big Endian)
  */
-static const char ncmagic[] = {'C', 'D', 'F', 0x01}; 
+static const char ncmagic[] = {'C', 'D', 'F', 0x01};
 
 #define ABSENT 0
 
@@ -231,7 +234,7 @@ swap8b(unsigned long long *val)
 static unsigned long long
 get_uint64(bufferinfo *gbp)
 {
-    /* retrieve a 64bit unisgned integer and return it as unsigned long long */
+    /* retrieve a 64bit unsigned integer and return it as unsigned long long */
     unsigned long long tmp;
     memcpy(&tmp, gbp->pos, 8);
     if (gbp->is_little_endian) swap8b(&tmp);
@@ -242,7 +245,7 @@ get_uint64(bufferinfo *gbp)
 static unsigned int
 get_uint32(bufferinfo *gbp)
 {
-    /* retrieve a 32bit unisgned integer and return it as unsigned int */
+    /* retrieve a 32bit unsigned integer and return it as unsigned int */
     unsigned int tmp;
     memcpy(&tmp, gbp->pos, 4);
     if (gbp->is_little_endian) swap4b(&tmp);
@@ -701,11 +704,11 @@ var_shape64(NC_var            *varp,
 out :
     /*
      * For CDF-1 and CDF-2 formats, the total number of array elements
-     * cannot exceed 2^32, unless this variable is the last fixed-size 
-     * variable, there is no record variable, and the file starting 
-     * offset of this variable is less than 2GiB. 
-     * We will check this in ncmpi_enddef() which calls ncmpii_NC_enddef() 
-     * which calls ncmpii_NC_check_vlens() 
+     * cannot exceed 2^32, unless this variable is the last fixed-size
+     * variable, there is no record variable, and the file starting
+     * offset of this variable is less than 2GiB.
+     * We will check this in ncmpi_enddef() which calls ncmpii_NC_enddef()
+     * which calls ncmpii_NC_check_vlens()
     if (ncp->format != 5 && product >= X_UINT_MAX)
         DEBUG_RETURN_ERROR(NC_EVARSIZE);
      */
@@ -799,7 +802,7 @@ compute_var_shape(NC *ncp)
     }
     else if (ncp->xsz > ncp->begin_var) {
         if (verbose) printf("Error:\n");
-        if (verbose) printf("\tfile header size (%lld) is larger than the begin of daat section (%lld)\n",ncp->xsz, ncp->begin_var);
+        if (verbose) printf("\tfile header size (%lld) is larger than the begin of data section (%lld)\n",ncp->xsz, ncp->begin_var);
         DEBUG_RETURN_ERROR(NC_ENOTNC) /* not a netCDF file or corrupted */
     }
     else if (ncp->begin_rec <= 0) {
@@ -855,13 +858,13 @@ val_fetch(int fd, bufferinfo *gbp) {
     size_t pos_addr, base_addr;
 
     assert(gbp->base != NULL);
-  
+
     pos_addr = (size_t) gbp->pos;
     base_addr = (size_t) gbp->base;
 
     slack = gbp->size - (pos_addr - base_addr);
     /* if gbp->pos and gbp->base are the same, there is no leftover buffer data
-     * to worry about.  
+     * to worry about.
      * In the other extreme, where gbp->size == (gbp->pos - gbp->base), then all
      * data in the buffer has been consumed */
     if (slack == gbp->size) slack = 0;
@@ -900,7 +903,7 @@ val_check_buffer(int         fd,
         return NC_NOERR;
 
     return val_fetch(fd, gbp);
-} 
+}
 
 static int
 val_get_NC_tag(int fd, bufferinfo *gbp, NC_tag *tagp, const char *loc)
@@ -943,11 +946,11 @@ hdr_get_NON_NEG(int fd, bufferinfo *gbp, long long *sp)
      */
     int sizeof_NON_NEG, status;
 
-    sizeof_NON_NEG = (gbp->version < 5) ? 4 : 8; 
+    sizeof_NON_NEG = (gbp->version < 5) ? 4 : 8;
     status = val_check_buffer(fd, gbp, sizeof_NON_NEG);
     if (status != NC_NOERR) {
         if (verbose) printf("%d-byte size is expected for ", sizeof_NON_NEG);
-        return status; 
+        return status;
     }
     if (gbp->version < 5)
         *sp = (long long) get_uint32(gbp);
@@ -1002,11 +1005,11 @@ hdr_get_name(int          fd,
 
     while (nchars > 0) {
         if (bufremain > 0) {
-            strcount = MIN(bufremain, nchars); 
+            strcount = MIN(bufremain, nchars);
             (void) memcpy(cpos, gbp->pos, strcount);
             nchars -= strcount;
             gbp->pos = (void *)((char *)gbp->pos + strcount);
-            cpos += strcount; 
+            cpos += strcount;
             bufremain -= strcount;
         } else {
             err_addr = ERR_ADDR;
@@ -1017,7 +1020,7 @@ hdr_get_name(int          fd,
                 free(*namep);
                 *namep = NULL;
                 return err;
-            } 
+            }
             bufremain = gbp->size;
         }
     }
@@ -1029,7 +1032,7 @@ hdr_get_name(int          fd,
             if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
             if (verbose) printf("\t%s - fetching name string padding\n", loc);
             return err;
-        } 
+        }
         memset(pad, 0, X_ALIGN-1);
         if (memcmp(gbp->pos, pad, padding) != 0) {
             /* This is considered not a fatal error, we continue to validate */
@@ -1051,10 +1054,11 @@ hdr_get_name(int          fd,
 }
 
 static int
-val_get_NC_dim(int fd, bufferinfo *gbp, NC_dim **dimpp) {
+val_get_NC_dim(int fd, bufferinfo *gbp, NC_dim **dimpp, NC_dimarray *ncap) {
     int err, status=NC_NOERR;
     char *name=NULL;
     size_t err_addr;
+    long long dim_length;
     NC_dim *dimp;
 
     *dimpp = NULL;
@@ -1065,24 +1069,32 @@ val_get_NC_dim(int fd, bufferinfo *gbp, NC_dim **dimpp) {
         return status;
     }
 
+    /* read dimension length */
+    err_addr = ERR_ADDR;
+    err = hdr_get_NON_NEG(fd, gbp, &dim_length);
+    if (err != NC_NOERR) { /* frees dimp */
+        if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
+        if (verbose) printf("\tDimension \"%s\": Failed to read dimension size\n",name);
+        free(name);
+        return err;
+    }
+
+    /* check if unlimited_id already set */
+    if (ncap->unlimited_id != -1 && dim_length == 0) {
+        if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
+        if (verbose) printf("\tDimension \"%s\": NC_UNLIMITED dimension already found (\"%s\")\n",name,ncap->value[ncap->unlimited_id]->name);
+        free(name);
+        return NC_EUNLIMIT;
+    }
+
     dimp = (NC_dim*) malloc(sizeof(NC_dim));
     if (dimp == NULL) {
-        if (name != NULL) free(name);
+        free(name);
         DEBUG_RETURN_ERROR(NC_ENOMEM)
     }
     dimp->name     = name;
     dimp->name_len = strlen(name);
-
-    /* read dimension size */
-    err_addr = ERR_ADDR;
-    err = hdr_get_NON_NEG(fd, gbp, &dimp->size);
-    if (err != NC_NOERR) { /* frees dimp */
-        if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
-        if (verbose) printf("\tDimension \"%s\": Failed to read dimenion size\n",name);
-        free(dimp->name);
-        free(dimp);
-        return err;
-    }
+    dimp->size     = dim_length;
 
     *dimpp = dimp;
 
@@ -1090,7 +1102,7 @@ val_get_NC_dim(int fd, bufferinfo *gbp, NC_dim **dimpp) {
 }
 
 static int
-val_get_NC_dimarray(int fd, bufferinfo *gbp, NC_dimarray *ncap)
+val_get_NC_dimarray(int fd, bufferinfo *gbp, NC_dimarray *ncap, long long numrecs)
 {
     /* netCDF file format:
      *  ...
@@ -1105,7 +1117,7 @@ val_get_NC_dimarray(int fd, bufferinfo *gbp, NC_dimarray *ncap)
      *                <non-negative INT64>        // CDF-5
      */
     int dim, err, status=NC_NOERR;
-    NC_tag tag = NC_UNSPECIFIED; 
+    NC_tag tag = NC_UNSPECIFIED;
     size_t tag_err_addr, nelems_err_addr;
     long long tmp;
 
@@ -1116,7 +1128,7 @@ val_get_NC_dimarray(int fd, bufferinfo *gbp, NC_dimarray *ncap)
     /* read NC_tag (NC_DIMENSION or ZERO) from gbp buffer */
     tag_err_addr = ERR_ADDR;
     err = val_get_NC_tag(fd, gbp, &tag, "tag NC_DIMENSION");
-    if (err != NC_NOERR) return err; 
+    if (err != NC_NOERR) return err;
 
     /* read nelems (number of dimensions) from gbp buffer */
     nelems_err_addr = ERR_ADDR;
@@ -1133,6 +1145,9 @@ val_get_NC_dimarray(int fd, bufferinfo *gbp, NC_dimarray *ncap)
         DEBUG_RETURN_ERROR(NC_EMAXDIMS)
     }
     ncap->ndefined = (int)tmp;
+    if (trace) printf("\ndim_list (dimension list):\n");
+
+    ncap->unlimited_id = -1;
 
     if (ncap->ndefined == 0) {
         /* no dimension defined */
@@ -1140,6 +1155,7 @@ val_get_NC_dimarray(int fd, bufferinfo *gbp, NC_dimarray *ncap)
          * NC_DIMENSION or ABSENT (ZERO), but we follow NetCDF library to skip
          * checking the tag when ndefined is zero.
          */
+        if (trace) printf("\ttag = ABSENT (no dimension defined)\n");
         return NC_NOERR;
 #if 0
         if (tag != ABSENT) {
@@ -1154,6 +1170,10 @@ val_get_NC_dimarray(int fd, bufferinfo *gbp, NC_dimarray *ncap)
             if (verbose) printf("\tInvalid NC component tag (%d), expecting NC_DIMENSION (%d)\n",tag,NC_DIMENSION);
             DEBUG_RETURN_ERROR(NC_ENOTNC)
         }
+        if (trace) {
+            printf("\ttag = NC_DIMENSION\n");
+            printf("\tnumber of dimensions defined = %d\n", ncap->ndefined);
+        }
 
         /* check each dimension */
         size_t alloc_size = (size_t)ncap->ndefined + NC_ARRAY_GROWBY;
@@ -1161,13 +1181,24 @@ val_get_NC_dimarray(int fd, bufferinfo *gbp, NC_dimarray *ncap)
         if (ncap->value == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
 
         for (dim=0; dim<ncap->ndefined; dim++) {
-            err = val_get_NC_dim(fd, gbp, &ncap->value[dim]);
+            err = val_get_NC_dim(fd, gbp, &ncap->value[dim], ncap);
             if (err != NC_NOERR && err != NC_ENULLPAD) {
                 ncap->ndefined = dim;
                 free_NC_dimarray(ncap);
                 return err;
             }
             if (status == NC_NOERR) status = err;
+            if (ncap->value[dim]->size == NC_UNLIMITED)
+                ncap->unlimited_id = dim; /* ID of unlimited dimension */
+
+            if (trace) {
+                if (ncap->unlimited_id == dim)
+                    printf("\tdimension ID %2d: name \"%s\", length = UNLIMITED (%lld currently)\n",
+                           dim,ncap->value[dim]->name, numrecs);
+                else
+                    printf("\tdimension ID %2d: name \"%s\", length = %lld\n",
+                           dim,ncap->value[dim]->name,ncap->value[dim]->size);
+            }
         }
     }
 
@@ -1215,7 +1246,7 @@ err_exit:
 }
 
 /*
- * Get the values of an attribute 
+ * Get the values of an attribute
  */
 static int
 val_get_NC_attrV(int         fd,
@@ -1225,7 +1256,7 @@ val_get_NC_attrV(int         fd,
 {
     int status=NC_NOERR;
     void *value = attrp->xvalue;
-    char pad[X_ALIGN-1]; 
+    char pad[X_ALIGN-1];
     size_t err_addr;
     long long nvalues, padding, bufremain, attcount;
     size_t pos_addr, base_addr;
@@ -1255,7 +1286,7 @@ val_get_NC_attrV(int         fd,
             bufremain = gbp->size;
         }
     }
- 
+
     if (padding > 0) {
         memset(pad, 0, X_ALIGN-1);
         if (memcmp(gbp->pos, pad, padding) != 0) {
@@ -1275,6 +1306,25 @@ val_get_NC_attrV(int         fd,
     return status;
 }
 
+static char*
+str_NC_type(nc_type xtype)
+{
+    switch(xtype) {
+        case NC_BYTE:   return "NC_BYTE";
+        case NC_CHAR:   return "NC_CHAR";
+        case NC_UBYTE:  return "NC_UBYTE";
+        case NC_SHORT:  return "NC_SHORT";
+        case NC_USHORT: return "NC_USHORT";
+        case NC_INT:    return "NC_INT";
+        case NC_UINT:   return "NC_UINT";
+        case NC_FLOAT:  return "NC_FLOAT";
+        case NC_DOUBLE: return "NC_DOUBLE";
+        case NC_INT64:  return "NC_INT64";
+        case NC_UINT64: return "NC_UINT64";
+        default: return "";
+    }
+}
+
 static long long
 x_len_NC_attrV(nc_type    xtype,
                long long nelems)
@@ -1326,15 +1376,15 @@ new_NC_attr(char        *name,
 }
 
 static int
-val_get_NC_attr(int          fd, 
-                bufferinfo  *gbp, 
+val_get_NC_attr(int          fd,
+                bufferinfo  *gbp,
                 NC_attr    **attrpp,
                 const char  *loc)
 {
     char *name=NULL, xloc[1024];
     int err, status=NC_NOERR;
     size_t err_addr;
-    nc_type xtype; 
+    nc_type xtype;
     long long nelems;
     NC_attr *attrp;
 
@@ -1352,7 +1402,7 @@ val_get_NC_attr(int          fd,
     }
 
     err_addr = ERR_ADDR;
-    err = hdr_get_NON_NEG(fd, gbp, &nelems); 
+    err = hdr_get_NON_NEG(fd, gbp, &nelems);
     if (err != NC_NOERR) {
         if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
         if (verbose) printf("\t%s: Failed to read attribute length\n",xloc);
@@ -1375,9 +1425,9 @@ val_get_NC_attr(int          fd,
     }
     if (status == NC_NOERR) status = err;
 
-    *attrpp = attrp; 
-  
-    return status; 
+    *attrpp = attrp;
+
+    return status;
 }
 
 static int
@@ -1411,7 +1461,7 @@ val_get_NC_attrarray(int           fd,
     /* read NC_tag (NC_ATTRIBUTE or ZERO) from gbp buffer */
     tag_err_addr = ERR_ADDR;
     err = val_get_NC_tag(fd, gbp, &tag, "tag NC_ATTRIBUTE");
-    if (err != NC_NOERR) return err; 
+    if (err != NC_NOERR) return err;
 
     /* read nelems (number of attributes) from gbp buffer */
     nelems_err_addr = ERR_ADDR;
@@ -1429,12 +1479,23 @@ val_get_NC_attrarray(int           fd,
     }
     ncap->ndefined = (int)tmp;
 
+    if (trace) {
+        if (!strcmp(loc, "Global"))
+            printf("\ngatt_list (global attribute list):\n");
+        else
+            printf("\t\tvatt_list (variable attribute list):\n");
+    }
+
     if (ncap->ndefined == 0) {
         /* no attribute defined */
         /* From the CDF file format specification, the tag is either
          * NC_ATTRIBUTE or ABSENT (ZERO), but we follow NetCDF library to skip
          * checking the tag when ndefined is zero.
          */
+        if (trace) {
+            if (strcmp(loc, "Global")) printf("\t\t");
+            printf("\ttag = ABSENT (no attribute defined)\n");
+        }
         return NC_NOERR;
 #if 0
         if (tag != ABSENT) {
@@ -1450,6 +1511,12 @@ val_get_NC_attrarray(int           fd,
             if (verbose) printf("\t%s: Invalid NC component tag (%d), expecting NC_ATTRIBUTE (%d)\n",xloc,tag,NC_ATTRIBUTE);
             DEBUG_RETURN_ERROR(NC_ENOTNC)
         }
+        if (trace) {
+            if (strcmp(loc, "Global")) printf("\t\t");
+            printf("\ttag = NC_ATTRIBUTE\n");
+            if (strcmp(loc, "Global")) printf("\t\t");
+            printf("\tnumber of attributes = %d\n", ncap->ndefined);
+        }
 
         size_t alloc_size = (size_t)ncap->ndefined + NC_ARRAY_GROWBY;
         ncap->value = (NC_attr **) calloc(alloc_size, sizeof(NC_attr *));
@@ -1463,9 +1530,15 @@ val_get_NC_attrarray(int           fd,
                 return err;
             }
             if (status == NC_NOERR) status = err;
+            if (trace) {
+                if (strcmp(loc, "Global")) printf("\t\t");
+                printf("\tattribute name \"%s\", type = %s, length = %lld\n",
+                       ncap->value[i]->name, str_NC_type(ncap->value[i]->xtype),
+                       ncap->value[i]->nelems);
+            }
         }
     }
-  
+
     return status;
 }
 
@@ -1552,10 +1625,12 @@ val_get_NC_var(int          fd,
     }
     status = err;
 
+    if (trace) printf("\t\tname = \"%s\"\n", name);
+
     /* read number of dimensions */
     sprintf(xloc,"%s \"%s\"",loc,name);
     err_addr = ERR_ADDR;
-    err = hdr_get_NON_NEG(fd, gbp, &ndims); 
+    err = hdr_get_NON_NEG(fd, gbp, &ndims);
     if (err != NC_NOERR) {
         if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
         if (verbose) printf("\t%s: Failed to read number of dimensions\n",xloc);
@@ -1572,6 +1647,8 @@ val_get_NC_var(int          fd,
         DEBUG_RETURN_ERROR(NC_EMAXDIMS)
     }
 
+    if (trace) printf("\t\tnumber of dimensions = %lld\n", ndims);
+
     /* allocate variable object */
     varp = val_new_NC_var(name, ndims);
     if (varp == NULL) {
@@ -1579,10 +1656,13 @@ val_get_NC_var(int          fd,
         DEBUG_RETURN_ERROR(NC_ENOMEM)
     }
 
+    if (trace && ndims > 0) printf("\t\tdimension IDs:");
+
     /* read dimension IDs and check dimensions */
     for (dim=0; dim<ndims; dim++) {
         err = val_check_buffer(fd, gbp, (gbp->version < 5 ? 4 : 8));
         if (err != NC_NOERR) {
+            if (trace) printf("\n");
             if (verbose) printf("Error @ [0x%8.8Lx]:\n", ERR_ADDR);
             if (verbose) printf("\t%s: Fail to read dimid[%d]\n",xloc,dim);
             free_NC_var(varp);
@@ -1594,8 +1674,11 @@ val_get_NC_var(int          fd,
         else
             dimid = (int) get_uint64(gbp);
 
+        if (trace) printf(" %d", dimid);
+
         /* dimid should be < f_ndims (num of dimensions defined in file) */
         if (dimid >= f_ndims) {
+            if (trace) printf("\n");
             if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
             if (verbose) printf("\t%s \"%s\": dimid[%d]=%d is larger than the number of dimensions defined in file (%d)\n",loc,name,dim,dimid,f_ndims);
             free_NC_var(varp);
@@ -1603,6 +1686,7 @@ val_get_NC_var(int          fd,
         }
         varp->dimids[dim] = dimid;
     }
+    if (trace && ndims > 0) printf("\n");
 
     /* var = name nelems [dimid ...] vatt_list nc_type vsize begin
      *                               ^^^^^^^^^                     */
@@ -1620,12 +1704,13 @@ val_get_NC_var(int          fd,
     if (err != NC_NOERR) {
         free_NC_var(varp);
         return err;
-    } 
+    }
+    if (trace) printf("\t\tdata type: %s\n", str_NC_type(varp->xtype));
 
     varp->xsz = len_nctype(varp->xtype);
 
     /* var = name nelems [dimid ...] vatt_list nc_type vsize begin
-     *                                                 ^^^^^      
+     *                                                 ^^^^^
      * instead of use vsize from file, we recalculate it in
      * compute_var_shape() */
     err_addr = ERR_ADDR;
@@ -1636,6 +1721,7 @@ val_get_NC_var(int          fd,
         free_NC_var(varp);
         return err;
     }
+    if (trace) printf("\t\tvariable size (vsize): %lld\n", varp->len);
 
     err = val_check_buffer(fd, gbp, (gbp->version == 1 ? 4 : 8));
     if (err != NC_NOERR) {
@@ -1651,6 +1737,8 @@ val_get_NC_var(int          fd,
     else
         varp->begin = (long long) get_uint64(gbp);
 
+    if (trace) printf("\t\tstarting file offset (begin): %lld\n", varp->begin);
+
     *varpp = varp;
     return status;
 }
@@ -1683,13 +1771,13 @@ val_get_NC_vararray(int          fd,
 
     assert(gbp != NULL && gbp->pos != NULL);
     assert(ncap != NULL);
-    assert(ncap->value == NULL); 
+    assert(ncap->value == NULL);
 
     /* read NC_tag (NC_VARIABLE or ZERO) from gbp buffer */
     tag_err_addr = ERR_ADDR;
     err = val_get_NC_tag(fd, gbp, &tag, "tag NC_VARIABLE");
     if (err != NC_NOERR) return err;
- 
+
     /* read nelems (number of variables) from gbp buffer */
     nelems_err_addr = ERR_ADDR;
     err = hdr_get_NON_NEG(fd, gbp, &tmp);
@@ -1706,11 +1794,14 @@ val_get_NC_vararray(int          fd,
     }
     ncap->ndefined = (int)tmp;
 
+    if (trace) printf("\nvar_list (variable list):\n");
+
     if (ncap->ndefined == 0) {
         /* From the CDF file format specification, the tag is either
          * NC_VARIABLE or ABSENT (ZERO), but we follow NetCDF library to skip
          * checking the tag when ndefined is zero.
          */
+        if (trace) printf("\ntag = ABSENT (no variable defined)\n");
         return NC_NOERR;
 #if 0
         if (tag != ABSENT) {
@@ -1725,12 +1816,15 @@ val_get_NC_vararray(int          fd,
             if (verbose) printf("\tInvalid NC component tag (%d), expecting NC_VARIABLE (%d)\n",tag,NC_VARIABLE);
             DEBUG_RETURN_ERROR(NC_ENOTNC)
         }
- 
+        if (trace) printf("\ttag = NC_VARIABLE\n");
+        if (trace) printf("\tnumber of variables = %d\n", ncap->ndefined);
+
         size_t alloc_size = (size_t)ncap->ndefined + NC_ARRAY_GROWBY;
         ncap->value = (NC_var **) calloc(alloc_size, sizeof(NC_var *));
-        if (ncap->value == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM) 
+        if (ncap->value == NULL) DEBUG_RETURN_ERROR(NC_ENOMEM)
 
         for (var=0; var<ncap->ndefined; var++) {
+            if (trace) printf("\n\tvariable ID = %d\n", var);
             err = val_get_NC_var(fd, gbp, &ncap->value[var], f_ndims, "Variable");
             if (err != NC_NOERR && err != NC_ENULLPAD) {
                 ncap->ndefined = var;
@@ -1954,6 +2048,7 @@ val_get_NC(int fd, NC *ncp)
     bufferinfo getbuf;
     char magic[5];
     size_t err_addr, pos_addr, base_addr;
+    const char *hdf5_signature="\211HDF\r\n\032\n";
 
     /* find Endianness of the running machine */
     getbuf.is_little_endian = check_little_endian();
@@ -1971,8 +2066,18 @@ val_get_NC(int fd, NC *ncp)
      */
     status = val_fetch(fd, &getbuf);
     if (status != NC_NOERR) goto fn_exit;
-  
-    /* First get the file format information, magic */
+
+    /* Check HDF file signature */
+    if (memcmp(getbuf.base, hdf5_signature, 8) == 0) {
+        if (verbose) {
+            printf("Error: Input file is in HDF format\n");
+            printf("       ncvalidator only validates NetCDF classic files\n");
+        }
+        status = NC_ENOTSUPPORT;
+        goto fn_exit;
+    }
+
+    /* Check classic CDF file signature */
     magic[4] = '\0';
     memcpy(magic, getbuf.base, 4);
     getbuf.pos = (char*)getbuf.pos + 4;
@@ -2000,6 +2105,7 @@ val_get_NC(int fd, NC *ncp)
         status = NC_ENOTNC;
         goto fn_exit;
     }
+    if (trace) printf("magic = %d (file format: CDF-%d)\n", ncp->format,ncp->format);
 
     /* header = magic numrecs dim_list gatt_list var_list
      * Check numrecs
@@ -2019,6 +2125,8 @@ val_get_NC(int fd, NC *ncp)
     else
         ncp->numrecs = (long long) get_uint64(&getbuf);
 
+    if (trace) printf("\nnumrecs = %lld (number of records)\n", ncp->numrecs);
+
     pos_addr  = (size_t) getbuf.pos;
     base_addr = (size_t) getbuf.base;
     assert(pos_addr < base_addr + getbuf.size);
@@ -2027,7 +2135,7 @@ val_get_NC(int fd, NC *ncp)
      * dim_list = ABSENT | NC_DIMENSION  nelems  [dim ...]
      * Check dim_list
      */
-    err = val_get_NC_dimarray(fd, &getbuf, &ncp->dims);
+    err = val_get_NC_dimarray(fd, &getbuf, &ncp->dims, ncp->numrecs);
     if (err != NC_NOERR && err != NC_ENULLPAD) {
         status = err;
         goto fn_exit;
@@ -2038,7 +2146,7 @@ val_get_NC(int fd, NC *ncp)
      * att_list = ABSENT | NC_ATTRIBUTE  nelems  [attr ...]
      * Check att_list
      */
-    err = val_get_NC_attrarray(fd, &getbuf, &ncp->attrs, "Global"); 
+    err = val_get_NC_attrarray(fd, &getbuf, &ncp->attrs, "Global");
     if (err != NC_NOERR && err != NC_ENULLPAD) {
         status = err;
         goto fn_exit;
@@ -2056,6 +2164,8 @@ val_get_NC(int fd, NC *ncp)
     }
     if (status == NC_NOERR) status = err;
 
+    if (trace) printf("\n");
+
     /* get file header size */
     ncp->xsz = hdr_len_NC(ncp);
 
@@ -2090,48 +2200,47 @@ static void
 usage(char *argv0)
 {
     char *help =
-    "Usage: %s [-h] | [-q] file_name\n"
+    "Usage: %s [-h] | [-t] [-x] [-q] file\n"
     "       [-h] Print help\n"
-    "       [-x] repair null-byte padding in file header\n"
+    "       [-t] Turn on tracing mode, printing progress of validation\n"
+    "       [-x] Repair in-place the null-byte padding in file header.\n"
     "       [-q] Quiet mode (exit 1 when fail, 0 success)\n"
-    "       filename: input netCDF file name\n";
+    "       file: Input netCDF file name\n"
+    "*PnetCDF library version PNETCDF_RELEASE_VERSION of PNETCDF_RELEASE_DATE\n";
     fprintf(stderr, help, argv0);
-    fprintf(stderr,"       PnetCDF library version 1.9.0\n");
 }
 
 int main(int argc, char **argv)
 {
     extern int optind;
-    char filename[512], *cmd, *path;
+    char filename[512], *path;
     int i, omode, fd, status=NC_NOERR;
     NC *ncp=NULL;
     struct stat ncfilestat;
 
-    cmd = (char*) malloc(strlen(argv[0])+1);
-    strcpy(cmd,argv[0]);
-
     /* get command-line arguments */
     verbose = 1;
+    trace = 0;
     repair  = 0;
-    while ((i = getopt(argc, argv, "xhq")) != EOF)
+    while ((i = getopt(argc, argv, "xthq")) != EOF)
         switch(i) {
             case 'x': repair = 1;
                       break;
+            case 't': trace = 1;
+                      break;
             case 'q': verbose = 0;
                       break;
             case 'h':
-            default:  usage(cmd);
-                      free(cmd);
+            default:  usage(argv[0]);
                       return 1;
         }
-    argc -= optind;
-    argv += optind;
-    if (argc != 1) {
-        usage(cmd);
-        free(cmd);
+
+    if (argv[optind] == NULL) { /* input file name is mandatory */
+        usage(argv[0]);
         return 1;
     }
-    snprintf(filename, 512, "%s", argv[0]);
+
+    snprintf(filename, 512, "%s", argv[optind]);
 
     /* remove the file system type prefix name if there is any.
      * For example, when filename = "lustre:/home/foo/testfile.nc", remove
@@ -2155,7 +2264,7 @@ int main(int argc, char **argv)
     ncp = (NC*) calloc(1, sizeof(NC));
     if (ncp == NULL) {
         status = NC_ENOMEM;
-        if (verbose) printf("Error at line %d when calling ncmpio_new_NC()\n",__LINE__);
+        if (verbose) printf("Error at line %d when calling calloc()\n",__LINE__);
         goto prog_exit;
     }
 
@@ -2223,16 +2332,15 @@ prog_exit:
 
     if (verbose) {
         if (status == NC_NOERR)
-            printf("File \"%s\" is a valid NetCDF file.\n",filename);
+            printf("File \"%s\" is a valid NetCDF classic file.\n",filename);
         else {
-            printf("File \"%s\" fails to conform with CDF file format specifications\n",filename);
+            printf("File \"%s\" fails to conform with classic CDF file format specifications\n",filename);
             if (repair) {
                 printf("and it has been repaired in place to remove the errors.\n");
-                printf("Please run \"%s %s\" to validate again.\n",cmd,filename);
+                printf("Please run \"%s %s\" to validate again.\n",argv[0],filename);
             }
         }
     }
-    free(cmd);
 
     exit((status == NC_NOERR) ? EXIT_SUCCESS : EXIT_FAILURE);
 }
