@@ -2,7 +2,7 @@
  *  Copyright (C) 2013, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: cdf_type.c 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 /* This program tests if PnetCDF can detect CDF-5 data types that are not
  * available in CDF-1 and CDF-2 formats
@@ -69,7 +69,7 @@ int test_var_types(char *filename,
         sprintf(name, "var_%d", i);
         err = ncmpi_def_var(ncid, name, xtype[i], 1, &dimid, &varid[i]);
         if (err != NC_ESTRICTCDF2) {
-            printf("Error (line=%d): expecting NC_ESTRICTCDF2 but got %s\n", __LINE__,nc_err_code_name(err));
+            printf("Error (line=%d): expecting NC_ESTRICTCDF2 but got %s\n", __LINE__,ncmpi_strerrno(err));
             nerrs++;
         }
     }
