@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: large_var.c 3622 2017-12-19 00:55:10Z wkliao $ */
+/* $Id$ */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
@@ -66,6 +66,7 @@ int main(int argc, char** argv)
     MPI_Offset offset, var_offset, start[3], count[3];
     MPI_File fh;
     MPI_Status status;
+    MPI_Info info;
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -88,34 +89,32 @@ int main(int argc, char** argv)
         free(cmd_str);
     }
 
+    MPI_Info_create(&info);
+    MPI_Info_set(info, "romio_ds_write", "disable");
+    MPI_Info_set(info, "romio_ds_read", "disable");
+
     /* create a new file for writing ----------------------------------------*/
     cmode = NC_CLOBBER | NC_64BIT_DATA;
-    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, info, &ncid);
     CHECK_ERR
 
     /* define dimensions Z, Y, and X */
-    err = ncmpi_def_dim(ncid, "Z", NZ, &dimid[0]);
-    CHECK_ERR
-    err = ncmpi_def_dim(ncid, "Y", NY, &dimid[1]);
-    CHECK_ERR
-    err = ncmpi_def_dim(ncid, "X", NX, &dimid[2]);
-    CHECK_ERR
+    err = ncmpi_def_dim(ncid, "Z", NZ, &dimid[0]); CHECK_ERR
+    err = ncmpi_def_dim(ncid, "Y", NY, &dimid[1]); CHECK_ERR
+    err = ncmpi_def_dim(ncid, "X", NX, &dimid[2]); CHECK_ERR
 
     /* define a big 1D variable of ubyte type, so there are 2 big variables */
     err = ncmpi_def_var(ncid, "big_1D", NC_UBYTE, 1, &dimid[2], &varid);
     CHECK_ERR
 
     /* define a big 3D variable of integer type */
-    err = ncmpi_def_var(ncid, "var", NC_INT, 3, dimid, &varid);
-    CHECK_ERR
+    err = ncmpi_def_var(ncid, "var", NC_INT, 3, dimid, &varid); CHECK_ERR
 
     /* do not forget to exit define mode */
-    err = ncmpi_enddef(ncid);
-    CHECK_ERR
+    err = ncmpi_enddef(ncid); CHECK_ERR
 
     /* get the beginning of file offset for the variable */
-    err = ncmpi_inq_varoffset(ncid, varid, &var_offset);
-    CHECK_ERR
+    err = ncmpi_inq_varoffset(ncid, varid, &var_offset); CHECK_ERR
 
     /* now we are in data mode */
     start[0] = 1;
@@ -128,41 +127,45 @@ int main(int argc, char** argv)
     buf = (int*) malloc(40 * sizeof(int));
     for (i=0; i<40; i++) buf[i] = rank*100 + i;
 
-    err = ncmpi_put_vara_int_all(ncid, varid, start, count, buf);
-    CHECK_ERR
+    /* write: subarray of 1 x 2 x 10 at start 1 x 8 x (2G + rank * 10) */
+    err = ncmpi_put_vara_int_all(ncid, varid, start, count, buf); CHECK_ERR
 
     /* now test nonblocking put */
+    /* write: subarray of 1 x 2 x 5 at start 3 x 8 x (2G + rank * 10) */
     start[0] = 3;
     count[2] = 5;
     err = ncmpi_iput_vara_int(ncid, varid, start, count, buf+20, &req[0]);
     CHECK_ERR
 
+    /* write: subarray of 1 x 1 x 5 at start 3 x 8 x (2G + rank * 10 + 5) */
     start[2] += 5;
     count[1]  = 1;
     err = ncmpi_iput_vara_int(ncid, varid, start, count, buf+30, &req[1]);
     CHECK_ERR
 
+    /* write: subarray of 1 x 1 x 5 at start 3 x 9 x (2G + rank * 10 + 5) */
     start[1] = 9;
     err = ncmpi_iput_vara_int(ncid, varid, start, count, buf+35, &req[2]);
     CHECK_ERR
 
-    err = ncmpi_wait_all(ncid, 3, req, st);
-    CHECK_ERR
+    err = ncmpi_wait_all(ncid, 3, req, st); CHECK_ERR
     for (i=0; i<3; i++) {
         err = st[i];
         CHECK_ERR
     }
 
-    err = ncmpi_close(ncid);
-    CHECK_ERR
+    err = ncmpi_close(ncid); CHECK_ERR
 
     /* open the same file and read back for validation */
-    err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, MPI_INFO_NULL,
-                     &ncid); CHECK_ERR
+    err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, info, &ncid); CHECK_ERR
 
     err = ncmpi_inq_varid(ncid, "var", &varid); CHECK_ERR
 
     /* read back subarray written by the process (rank+1)%nprocs */
+    int expected = (rank == nprocs - 1) ? 0 : (rank+1)*100;
+
+    /* read back ((rank+1)%nprocs)'s 1st write:
+     *                      subarray of 1x2x10 at start 1x8x(2G+rank*10) */
     start[0] = 1;
     start[1] = 8;
     start[2] = TWO_G + ((rank == nprocs - 1) ? 0 : 10 * (rank + 1));
@@ -176,8 +179,6 @@ int main(int argc, char** argv)
 
     err = ncmpi_get_vara_int_all(ncid, varid, start, count, buf); CHECK_ERR
 
-    int expected = (rank == nprocs - 1) ? 0 : (rank+1)*100;
-
     /* check if the contents of buf are expected */
     for (i=0; i<bufsize; i++) {
         if (buf[i] != expected) {
@@ -188,6 +189,8 @@ int main(int argc, char** argv)
         expected++;
     }
 
+    /* read back ((rank+1)%nprocs)'s 2nd write:
+     *                      subarray of 1x2x5 at start 3x8x(2G+rank*10) */
     start[0] = 3;
     count[2] = 5;
 
@@ -207,6 +210,8 @@ int main(int argc, char** argv)
         expected++;
     }
 
+    /* read back ((rank+1)%nprocs)'s 3rd write:
+                            subarray of 1x1x5 at start 3x8x(2G+rank*10+5) */
     start[2] += 5;
     count[1] = 1;
 
@@ -226,6 +231,8 @@ int main(int argc, char** argv)
         expected++;
     }
 
+    /* read back ((rank+1)%nprocs)'s 4th write:
+                            subarray of 1x1x5 at start 3x9x(2G+rank*10+5) */
     start[1] = 9;
 
     /* initialize the contents of the array to a different value */
@@ -379,6 +386,7 @@ int main(int argc, char** argv)
     MPI_File_close(&fh);
 
     free(buf);
+    MPI_Info_free(&info);
 
     /* check if PnetCDF freed all internal malloc */
     MPI_Offset malloc_size, sum_size;
