@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: i_varn_indef.c 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * This example tests posting nonblocking varn APIs, including
@@ -178,6 +178,8 @@ int main(int argc, char** argv)
     long long *buffer[4], *cbuffer[4], *rbuffer[4];
     int num_segs[4] = {4, 6, 5, 4};
     int req_lens[4], my_nsegs[4];
+    int bb_enabled=0;
+
     MPI_Datatype buftype[4];
     MPI_Offset **starts[4], **counts[4];
     MPI_Offset n_starts[4][6][2] = {{{0,5}, {1,0}, {2,6}, {3,0}, {0,0}, {0,0}},
@@ -190,25 +192,25 @@ int main(int argc, char** argv)
                                     {{1,3}, {1,1}, {1,3}, {1,3}, {0,0}, {0,0}}};
 
     /* n_starts[0][][] n_counts[0][][] indicate the following: ("-" means skip)
-              -  -  -  -  -  X  X  -  -  - 
-              X  -  -  -  -  -  -  -  -  - 
-              -  -  -  -  -  -  X  X  -  - 
-              X  X  X  -  -  -  -  -  -  - 
+              -  -  -  -  -  X  X  -  -  -
+              X  -  -  -  -  -  -  -  -  -
+              -  -  -  -  -  -  X  X  -  -
+              X  X  X  -  -  -  -  -  -  -
        n_starts[1][][] n_counts[1][][] indicate the following pattern.
-              -  -  -  X  X  -  -  -  X  X 
-              -  -  -  -  -  X  X  -  -  - 
-              X  X  -  -  -  -  -  -  X  X 
-              -  -  -  -  X  X  X  -  -  - 
+              -  -  -  X  X  -  -  -  X  X
+              -  -  -  -  -  X  X  -  -  -
+              X  X  -  -  -  -  -  -  X  X
+              -  -  -  -  X  X  X  -  -  -
        n_starts[2][][] n_counts[2][][] indicate the following pattern.
-              -  -  -  -  -  -  -  X  -  - 
-              -  X  X  X  -  -  -  X  X  X 
-              -  -  X  -  -  -  -  -  -  - 
-              -  -  -  X  -  -  -  -  -  - 
+              -  -  -  -  -  -  -  X  -  -
+              -  X  X  X  -  -  -  X  X  X
+              -  -  X  -  -  -  -  -  -  -
+              -  -  -  X  -  -  -  -  -  -
        n_starts[3][][] n_counts[3][][] indicate the following pattern.
-              X  X  X  -  -  -  -  -  -  - 
-              -  -  -  -  X  -  -  -  -  - 
-              -  -  -  X  X  X  -  -  -  - 
-              -  -  -  -  -  -  -  X  X  X 
+              X  X  X  -  -  -  -  -  -  -
+              -  -  -  -  X  -  -  -  -  -
+              -  -  -  X  X  X  -  -  -  -
+              -  -  -  -  -  -  -  X  X  X
      */
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -300,6 +302,18 @@ int main(int argc, char** argv)
     err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
     CHECK_ERR
 
+    {
+        int flag;
+        char hint[MPI_MAX_INFO_VAL];
+        MPI_Info infoused;
+
+        ncmpi_inq_file_info(ncid, &infoused);
+        MPI_Info_get(infoused, "nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
+        if (flag && strcasecmp(hint, "enable") == 0)
+            bb_enabled = 1;
+        MPI_Info_free(&infoused);
+    }
+
     /* create a global array of size NY * NX */
     err = ncmpi_def_dim(ncid, "Y", NY, &dimid[0]); CHECK_ERR
     err = ncmpi_def_dim(ncid, "X", NX, &dimid[1]); CHECK_ERR
@@ -319,7 +333,7 @@ int main(int argc, char** argv)
                                    NULL, &reqs[4]);
     if (err != NC_ENULLSTART) {
         printf("expecting error code NC_ENULLSTART but got %s\n",
-               nc_err_code_name(err));
+               ncmpi_strerrno(err));
         nerrs++;
     }
 
@@ -327,7 +341,10 @@ int main(int argc, char** argv)
 
     /* clear the file contents using a blocking API, before commit the
      * nonblocking requests posted in define mode */
-    nerrs += clear_file_contents(ncid, varid);
+    if (!bb_enabled) {
+        nerrs += clear_file_contents(ncid, varid);
+    }
+
     nerrs += check_num_pending_reqs(ncid, nreqs, __LINE__);
     err = ncmpi_wait_all(ncid, nreqs, reqs, sts);
     CHECK_ERR
@@ -378,7 +395,10 @@ int main(int argc, char** argv)
 
     /* clear the file contents using a blocking API, before commit the
      * nonblocking requests posted in define mode */
-    nerrs += clear_file_contents(ncid, varid);
+    if (!bb_enabled) {
+        nerrs += clear_file_contents(ncid, varid);
+    }
+
     nerrs += check_num_pending_reqs(ncid, nreqs, __LINE__);
     err = ncmpi_wait_all(ncid, nreqs, reqs, sts);
     CHECK_ERR
@@ -439,7 +459,10 @@ int main(int argc, char** argv)
 
     /* clear the file contents using a blocking API, before commit the
      * nonblocking requests posted in define mode */
-    nerrs += clear_file_contents(ncid, varid);
+    if (!bb_enabled) {
+        nerrs += clear_file_contents(ncid, varid);
+    }
+
     nerrs += check_num_pending_reqs(ncid, nreqs*2, __LINE__);
     err = ncmpi_wait_all(ncid, nreqs, reqs, sts);
     CHECK_ERR
@@ -534,7 +557,10 @@ int main(int argc, char** argv)
 
     /* clear the file contents using a blocking API, before commit the
      * nonblocking requests posted in define mode */
-    nerrs += clear_file_contents(ncid, varid);
+    if (!bb_enabled) {
+        nerrs += clear_file_contents(ncid, varid);
+    }
+
     nerrs += check_num_pending_reqs(ncid, nreqs*3, __LINE__);
     err = ncmpi_wait_all(ncid, nreqs, reqs, sts);
     CHECK_ERR
@@ -596,9 +622,11 @@ int main(int argc, char** argv)
     err = ncmpi_inq_malloc_size(&malloc_size);
     if (err == NC_NOERR) {
         MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
-        if (rank == 0 && sum_size > 0)
+        if (rank == 0 && sum_size > 0) {
             printf("heap memory allocated by PnetCDF internally has %lld bytes yet to be freed\n",
                    sum_size);
+            ncmpi_inq_malloc_list();
+        }
     }
 
     MPI_Allreduce(MPI_IN_PLACE, &nerrs, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
