@@ -4,7 +4,7 @@
  *  See COPYRIGHT notice in top-level directory.
  *
  *********************************************************************/
-/* $Id: i_varn_int64.c 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * This example tests nonblocking varn APIs, including
@@ -24,11 +24,16 @@
  *    dimensions:
  *             Y = 4 ;
  *             X = 10 ;
+ *    	       time = UNLIMITED ; // (4 currently)
  *    variables:
  *            int64 var0(Y, X) ;
  *            int64 var1(Y, X) ;
  *            int64 var2(Y, X) ;
  *            int64 var3(Y, X) ;
+ *            int64 t_var0(time, X) ;
+ *            int64 t_var1(time, X) ;
+ *            int64 t_var2(time, X) ;
+ *            int64 t_var3(time, X) ;
  *    data:
  *
  *     var0 =
@@ -54,6 +59,30 @@
  *      11, 13, 13, 13, 10, 12, 12, 13, 13, 13,
  *      12, 12, 13, 10, 10, 10, 11, 11, 12, 12,
  *      11, 11, 11, 13, 12, 12, 12, 10, 10, 10 ;
+ *
+ *     t_var0 =
+ *      13, 13, 13, 11, 11, 10, 10, 12, 11, 11,
+ *      10, 12, 12, 12, 13, 11, 11, 12, 12, 12,
+ *      11, 11, 12, 13, 13, 13, 10, 10, 11, 11,
+ *      10, 10, 10, 12, 11, 11, 11, 13, 13, 13 ;
+ *
+ *     t_var1 =
+ *      12, 12, 12, 10, 10, 13, 13, 11, 10, 10,
+ *      13, 11, 11, 11, 12, 10, 10, 11, 11, 11,
+ *      10, 10, 11, 12, 12, 12, 13, 13, 10, 10,
+ *      13, 13, 13, 11, 10, 10, 10, 12, 12, 12 ;
+ *
+ *     t_var2 =
+ *      11, 11, 11, 13, 13, 12, 12, 10, 13, 13,
+ *      12, 10, 10, 10, 11, 13, 13, 10, 10, 10,
+ *      13, 13, 10, 11, 11, 11, 12, 12, 13, 13,
+ *      12, 12, 12, 10, 13, 13, 13, 11, 11, 11 ;
+ *
+ *     t_var3 =
+ *      10, 10, 10, 12, 12, 11, 11, 13, 12, 12,
+ *      11, 13, 13, 13, 10, 12, 12, 13, 13, 13,
+ *      12, 12, 13, 10, 10, 10, 11, 11, 12, 12,
+ *      11, 11, 11, 13, 12, 12, 12, 10, 10, 10 ;
  *    }
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -92,6 +121,20 @@ int clear_file_contents(int ncid, int *varid)
 {
     int i, err, rank, nerrs=0;
     long long *w_buffer = (long long*) malloc(NY*NX * sizeof(long long));
+    int bb_enabled=0;
+
+    {
+        int flag;
+        char hint[MPI_MAX_INFO_VAL];
+        MPI_Info infoused;
+
+        ncmpi_inq_file_info(ncid, &infoused);
+        MPI_Info_get(infoused, "nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
+        if (flag && strcasecmp(hint, "enable") == 0)
+            bb_enabled = 1;
+        MPI_Info_free(&infoused);
+    }
+
     for (i=0; i<NY*NX; i++) w_buffer[i] = -1;
 
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -101,6 +144,13 @@ int clear_file_contents(int ncid, int *varid)
         CHECK_ERR
     }
     free(w_buffer);
+
+    // Flush the log to prevent new value being skipped due to overlaping domain
+    if (bb_enabled) {
+        err = ncmpi_flush(ncid);
+        CHECK_ERR
+    }
+
     return nerrs;
 }
 
@@ -176,11 +226,11 @@ void permute(MPI_Offset *a, MPI_Offset *b)
     }
 }
 
-int main(int argc, char** argv)
+static int
+test_varn(int ncid, int rank, int *varid)
 {
-    char filename[256];
-    int i, j, k, rank, nprocs, err, nerrs=0, bufsize=0;
-    int ncid, cmode, varid[4], dimid[2], nreqs, reqs[4], sts[4];
+    int i, j, k, err, nerrs=0, bufsize=0;
+    int nreqs, reqs[4], sts[4];
     long long *buffer[4], *cbuffer[4];
     int num_segs[4] = {4, 6, 5, 4};
     int req_lens[4], my_nsegs[4];
@@ -195,64 +245,26 @@ int main(int argc, char** argv)
                                     {{1,3}, {1,1}, {1,3}, {1,3}, {0,0}, {0,0}}};
 
     /* n_starts[0][][] n_counts[0][][] indicate the following: ("-" means skip)
-              -  -  -  -  -  X  X  -  -  - 
-              X  -  -  -  -  -  -  -  -  - 
-              -  -  -  -  -  -  X  X  -  - 
-              X  X  X  -  -  -  -  -  -  - 
+              -  -  -  -  -  X  X  -  -  -
+              X  -  -  -  -  -  -  -  -  -
+              -  -  -  -  -  -  X  X  -  -
+              X  X  X  -  -  -  -  -  -  -
        n_starts[1][][] n_counts[1][][] indicate the following pattern.
-              -  -  -  X  X  -  -  -  X  X 
-              -  -  -  -  -  X  X  -  -  - 
-              X  X  -  -  -  -  -  -  X  X 
-              -  -  -  -  X  X  X  -  -  - 
+              -  -  -  X  X  -  -  -  X  X
+              -  -  -  -  -  X  X  -  -  -
+              X  X  -  -  -  -  -  -  X  X
+              -  -  -  -  X  X  X  -  -  -
        n_starts[2][][] n_counts[2][][] indicate the following pattern.
-              -  -  -  -  -  -  -  X  -  - 
-              -  X  X  X  -  -  -  X  X  X 
-              -  -  X  -  -  -  -  -  -  - 
-              -  -  -  X  -  -  -  -  -  - 
+              -  -  -  -  -  -  -  X  -  -
+              -  X  X  X  -  -  -  X  X  X
+              -  -  X  -  -  -  -  -  -  -
+              -  -  -  X  -  -  -  -  -  -
        n_starts[3][][] n_counts[3][][] indicate the following pattern.
-              X  X  X  -  -  -  -  -  -  - 
-              -  -  -  -  X  -  -  -  -  - 
-              -  -  -  X  X  X  -  -  -  - 
-              -  -  -  -  -  -  -  X  X  X 
+              X  X  X  -  -  -  -  -  -  -
+              -  -  -  -  X  -  -  -  -  -
+              -  -  -  X  X  X  -  -  -  -
+              -  -  -  -  -  -  -  X  X  X
      */
-    MPI_Init(&argc, &argv);
-    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
-
-    if (argc > 2) {
-        if (!rank) printf("Usage: %s [filename]\n",argv[0]);
-        MPI_Finalize();
-        return 1;
-    }
-    if (argc == 2) snprintf(filename, 256, "%s", argv[1]);
-    else           strcpy(filename, "testfile.nc");
-    MPI_Bcast(filename, 256, MPI_CHAR, 0, MPI_COMM_WORLD);
-
-    if (rank == 0) {
-        char *cmd_str = (char*)malloc(strlen(argv[0]) + 256);
-        sprintf(cmd_str, "*** TESTING C   %s for iput/iget varn ", basename(argv[0]));
-        printf("%-66s ------ ", cmd_str);
-        free(cmd_str);
-    }
-
-#ifdef DEBUG
-    if (nprocs != 4 && rank == 0)
-        printf("Warning: %s is intended to run on 4 processes\n",argv[0]);
-#endif
-
-    /* create a new file for writing ----------------------------------------*/
-    cmode = NC_CLOBBER | NC_64BIT_DATA;
-    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
-    FATAL_ERR
-
-    /* create a global array of size NY * NX */
-    err = ncmpi_def_dim(ncid, "Y", NY, &dimid[0]); CHECK_ERR
-    err = ncmpi_def_dim(ncid, "X", NX, &dimid[1]); CHECK_ERR
-    err = ncmpi_def_var(ncid, "var0", NC_INT64, NDIMS, dimid, &varid[0]); CHECK_ERR
-    err = ncmpi_def_var(ncid, "var1", NC_INT64, NDIMS, dimid, &varid[1]); CHECK_ERR
-    err = ncmpi_def_var(ncid, "var2", NC_INT64, NDIMS, dimid, &varid[2]); CHECK_ERR
-    err = ncmpi_def_var(ncid, "var3", NC_INT64, NDIMS, dimid, &varid[3]); CHECK_ERR
-    err = ncmpi_enddef(ncid); CHECK_ERR
 
     /* allocate space for starts and counts */
     starts[0] = (MPI_Offset**) malloc(4 * 6 * sizeof(MPI_Offset*));
@@ -346,6 +358,8 @@ int main(int argc, char** argv)
         for (i=1; i<nreqs; i++) cbuffer[i] = cbuffer[i-1] + req_lens[i-1];
         for (i=0; i<bufsize; i++) cbuffer[0][i] = rank+10;
     }
+    else
+        for (i=0; i<4; i++) cbuffer[i] = NULL;
 
     /* write usning varn API */
     nerrs += clear_file_contents(ncid, varid);
@@ -414,6 +428,7 @@ int main(int argc, char** argv)
                                        counts[i], buffer[i], &reqs[i]);
         CHECK_ERR
     }
+
     nerrs += check_num_pending_reqs(ncid, nreqs, __LINE__);
     err = ncmpi_wait_all(ncid, nreqs, reqs, sts);
     CHECK_ERR
@@ -423,8 +438,8 @@ int main(int argc, char** argv)
     for (i=0; i<nreqs; i++) {
         for (j=0; j<req_lens[i]; j++) {
             if (buffer[i][j] != rank+10) {
-                printf("Error at line %d in %s: expecting buffer[%d][%d]=%d but got %lld\n",
-                       __LINE__,__FILE__,i,j,rank+10,buffer[i][j]);
+                printf("Error at line %d in %s: expecting var %d buffer[%d][%d]=%d but got %lld\n",
+                       __LINE__,__FILE__,varid[i],i,j,rank+10,buffer[i][j]);
                 nerrs++;
             }
         }
@@ -497,12 +512,15 @@ int main(int argc, char** argv)
         }
     }
 
-    /* read back using a contiguous buffer. First swap back the starts[] and counts[].
-     * swap segment 0 with segment 2 and swap segment 1 with segment 3
+    /* read back using a contiguous buffer. First swap back the starts[] and
+     * counts[]. swap segment 0 with segment 2 and swap segment 1 with segment
+     * 3
      */
     for (i=0; i<nreqs; i++) {
-        permute(starts[i][0], starts[i][2]); permute(counts[i][0], counts[i][2]);
-        permute(starts[i][1], starts[i][3]); permute(counts[i][1], counts[i][3]);
+        permute(starts[i][0], starts[i][2]);
+        permute(counts[i][0], counts[i][2]);
+        permute(starts[i][1], starts[i][3]);
+        permute(counts[i][1], counts[i][3]);
     }
 
     for (i=0; i<bufsize; i++) cbuffer[0][i] = -1;
@@ -528,9 +546,6 @@ int main(int argc, char** argv)
         }
     }
 
-    err = ncmpi_close(ncid);
-    CHECK_ERR
-
     if (bufsize>0) free(cbuffer[0]);
     for (i=0; i<nreqs; i++) free(buffer[i]);
     free(starts[0][0]);
@@ -538,14 +553,82 @@ int main(int argc, char** argv)
     free(starts[0]);
     free(counts[0]);
 
+    return nerrs;
+}
+
+int main(int argc, char** argv)
+{
+    char filename[256];
+    int rank, nprocs, err, nerrs=0;
+    int ncid, cmode, varid[4], dimid[2];
+
+    MPI_Init(&argc, &argv);
+    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
+    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
+
+    if (argc > 2) {
+        if (!rank) printf("Usage: %s [filename]\n",argv[0]);
+        MPI_Finalize();
+        return 1;
+    }
+    if (argc == 2) snprintf(filename, 256, "%s", argv[1]);
+    else           strcpy(filename, "testfile.nc");
+    MPI_Bcast(filename, 256, MPI_CHAR, 0, MPI_COMM_WORLD);
+
+    if (rank == 0) {
+        char *cmd_str = (char*)malloc(strlen(argv[0]) + 256);
+        sprintf(cmd_str, "*** TESTING C   %s for iput/iget varn ", basename(argv[0]));
+        printf("%-66s ------ ", cmd_str);
+        free(cmd_str);
+    }
+
+#ifdef DEBUG
+    if (nprocs != 4 && rank == 0)
+        printf("Warning: %s is intended to run on 4 processes\n",argv[0]);
+#endif
+
+    /* create a new file for writing ----------------------------------------*/
+    cmode = NC_CLOBBER | NC_64BIT_DATA;
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
+    FATAL_ERR
+
+    /* create fixed-size variables of size NY * NX */
+    err = ncmpi_def_dim(ncid, "Y", NY, &dimid[0]); CHECK_ERR
+    err = ncmpi_def_dim(ncid, "X", NX, &dimid[1]); CHECK_ERR
+    err = ncmpi_def_var(ncid, "var0", NC_INT64, NDIMS, dimid, &varid[0]); CHECK_ERR
+    err = ncmpi_def_var(ncid, "var1", NC_INT64, NDIMS, dimid, &varid[1]); CHECK_ERR
+    err = ncmpi_def_var(ncid, "var2", NC_INT64, NDIMS, dimid, &varid[2]); CHECK_ERR
+    err = ncmpi_def_var(ncid, "var3", NC_INT64, NDIMS, dimid, &varid[3]); CHECK_ERR
+    err = ncmpi_enddef(ncid); CHECK_ERR
+
+    /* test fixed-size variables */
+    nerrs += test_varn(ncid, rank, varid);
+
+    err = ncmpi_redef(ncid); CHECK_ERR
+
+    /* add record variables */
+    err = ncmpi_def_dim(ncid, "time", NC_UNLIMITED, &dimid[0]); CHECK_ERR
+    err = ncmpi_def_var(ncid, "t_var0", NC_INT64, NDIMS, dimid, &varid[0]); CHECK_ERR
+    err = ncmpi_def_var(ncid, "t_var1", NC_INT64, NDIMS, dimid, &varid[1]); CHECK_ERR
+    err = ncmpi_def_var(ncid, "t_var2", NC_INT64, NDIMS, dimid, &varid[2]); CHECK_ERR
+    err = ncmpi_def_var(ncid, "t_var3", NC_INT64, NDIMS, dimid, &varid[3]); CHECK_ERR
+    err = ncmpi_enddef(ncid); CHECK_ERR
+
+    /* test record variables */
+    nerrs += test_varn(ncid, rank, varid);
+
+    err = ncmpi_close(ncid); CHECK_ERR
+
     /* check if PnetCDF freed all internal malloc */
     MPI_Offset malloc_size, sum_size;
     err = ncmpi_inq_malloc_size(&malloc_size);
     if (err == NC_NOERR) {
         MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
-        if (rank == 0 && sum_size > 0)
+        if (rank == 0 && sum_size > 0) {
             printf("heap memory allocated by PnetCDF internally has %lld bytes yet to be freed\n",
                    sum_size);
+            ncmpi_inq_malloc_list();
+        }
     }
 
     MPI_Allreduce(MPI_IN_PLACE, &nerrs, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
