@@ -2,7 +2,7 @@
  *  Copyright (C) 2013, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  *
- *  $Id: flexible.c 3422 2017-09-25 05:42:52Z wkliao $
+ *  $Id$
  */
 
 /*
@@ -25,7 +25,7 @@
  *    variables:
  * 	   int VAR(Y, X) ;
  *    data:
- * 
+ *
  *    var =
  *      11, 11, 11, 11, 11,
  *      10, 10, 10, 10, 10 ;
@@ -48,7 +48,7 @@
 int main(int argc, char **argv) {
 
     char         filename[256];
-    int          i, j, err, ncid, varid, dimids[2], debug=0;
+    int          i, j, err, ncid, varid1, varid2, varid3, dimids[2], debug=0;
     int          rank, nprocs, blocklengths[2], buf[NY][NX], *bufptr;
     int         *ncbuf, req, st, nerrs=0;
     int          array_of_sizes[2], array_of_subsizes[2], array_of_starts[2];
@@ -82,7 +82,9 @@ int main(int argc, char **argv) {
     /* define a 2D array */
     err = ncmpi_def_dim(ncid, "Y", NC_UNLIMITED, &dimids[0]); CHECK_ERR
     err = ncmpi_def_dim(ncid, "X", NX*nprocs,    &dimids[1]); CHECK_ERR
-    err = ncmpi_def_var(ncid, "var", NC_INT, 2, dimids, &varid); CHECK_ERR
+    err = ncmpi_def_var(ncid, "var_int", NC_INT, 2, dimids, &varid1); CHECK_ERR
+    err = ncmpi_def_var(ncid, "var_dbl", NC_DOUBLE, 2, dimids, &varid2); CHECK_ERR
+    err = ncmpi_def_var(ncid, "var_byte", NC_BYTE, 2, dimids, &varid3); CHECK_ERR
     err = ncmpi_enddef(ncid); CHECK_ERR
 
     /* initialize the contents of the array */
@@ -104,17 +106,64 @@ int main(int argc, char **argv) {
     if (debug) printf("put start=%lld %lld count=%lld %lld\n",start[0],start[1],count[0],count[1]);
 
     /* call flexible API */
-    err = ncmpi_put_vara_all(ncid, varid, start, count, bufptr, 1, buftype); CHECK_ERR
-    MPI_Type_free(&buftype);
+    err = ncmpi_put_vara_all(ncid, varid1, start, count, bufptr, 1, buftype); CHECK_ERR
 
     /* check if the contents of buf are altered */
-    for (j=0; j<NY; j++)
-        for (i=0; i<NX; i++)
-            if (buf[j][i] != j+10)
-                printf("buf[%d][%d] != %d\n",j,i,buf[j][i]);
- 
+    for (j=0; j<NY; j++) for (i=0; i<NX; i++)
+        if (buf[j][i] != j+10) {
+            printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                   __FILE__,__LINE__,j,i,j+10,buf[j][i]);
+            nerrs++;
+        }
+
+    /* call flexible API that do type conversion */
+    err = ncmpi_put_vara_all(ncid, varid2, start, count, bufptr, 1, buftype); CHECK_ERR
+
+    /* check if the contents of buf are altered */
+    for (j=0; j<NY; j++) for (i=0; i<NX; i++)
+        if (buf[j][i] != j+10) {
+            printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                   __FILE__,__LINE__,j,i,j+10,buf[j][i]);
+            nerrs++;
+        }
+
+    MPI_Type_free(&buftype);
+
+    /* call flexible API that do no type conversion */
+    signed char *schar_buf = (signed char *) malloc(NY*NX);
+    for (j=0; j<NY; j++) for (i=0; i<NX; i++) schar_buf[j*NX+i] = j+10;
+    disps[0] = 0;
+    disps[1] = -NX;
+    err = MPI_Type_create_hindexed(2, blocklengths, disps, MPI_SIGNED_CHAR, &buftype);
+    if (err != MPI_SUCCESS) printf("MPI error MPI_Type_create_hindexed\n");
+    MPI_Type_commit(&buftype);
+
+    err = ncmpi_put_vara_all(ncid, varid3, start, count, schar_buf+NX, 1, buftype); CHECK_ERR
+
+    /* check if the contents of buf are altered */
+    for (j=0; j<NY; j++) for (i=0; i<NX; i++)
+        if (schar_buf[j*NX+i] != j+10) {
+            printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                   __FILE__,__LINE__,j,i,j+10,schar_buf[j*NX+i]);
+            nerrs++;
+        }
+
+    for (j=0; j<NY; j++) for (i=0; i<NX; i++) schar_buf[j*NX+i] = -1;
+    err = ncmpi_get_vara_all(ncid, varid3, start, count, schar_buf+NX, 1, buftype); CHECK_ERR
+
+    /* check read contents */
+    for (j=0; j<NY; j++) for (i=0; i<NX; i++)
+        if (schar_buf[j*NX+i] != j+10) {
+            printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                   __FILE__,__LINE__,j,i,j+10,schar_buf[j*NX+i]);
+            nerrs++;
+        }
+
+    free(schar_buf);
+    MPI_Type_free(&buftype);
+
     /* check if root process can write to file header in data mode */
-    err = ncmpi_rename_var(ncid, varid, "VAR"); CHECK_ERR
+    err = ncmpi_rename_var(ncid, varid1, "VAR"); CHECK_ERR
 
     err = ncmpi_close(ncid); CHECK_ERR
 
@@ -122,7 +171,7 @@ int main(int argc, char **argv) {
     err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, MPI_INFO_NULL,
                      &ncid); CHECK_ERR
 
-    err = ncmpi_inq_varid(ncid, "VAR", &varid); CHECK_ERR
+    err = ncmpi_inq_varid(ncid, "VAR", &varid1); CHECK_ERR
 
     /* initialize the contents of the array to a different value */
     for (j=0; j<NY; j++) for (i=0; i<NX; i++) buf[j][i] = -1;
@@ -132,14 +181,31 @@ int main(int argc, char **argv) {
     count[0] = 2; count[1] = NX;
     if (debug) printf("get start=%lld %lld count=%lld %lld\n",start[0],start[1],count[0],count[1]);
 
-    err = ncmpi_get_vara_int_all(ncid, varid, start, count, buf[0]); CHECK_ERR
+    err = ncmpi_get_vara_int_all(ncid, varid1, start, count, buf[0]); CHECK_ERR
 
     /* check if the contents of buf are expected */
     for (j=0; j<2; j++) {
-        int val = (j == 0) ? 1 : 0;
+        int expect = (j == 0) ? 1 : 0;
         for (i=0; i<NX; i++)
-            if (buf[j][i] != val+10) {
-                printf("Unexpected buf[%d][%d]=%d != %d\n",j,i,buf[j][i],val);
+            if (buf[j][i] != expect+10) {
+                printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                       __FILE__,__LINE__,j,i,expect+10,buf[j][i]);
+                nerrs++;
+            }
+    }
+
+    /* initialize the contents of the array to a different value */
+    for (j=0; j<NY; j++) for (i=0; i<NX; i++) buf[j][i] = -1;
+
+    err = ncmpi_get_vara_int_all(ncid, varid2, start, count, buf[0]); CHECK_ERR
+
+    /* check if the contents of buf are expected */
+    for (j=0; j<2; j++) {
+        int expect = (j == 0) ? 1 : 0;
+        for (i=0; i<NX; i++)
+            if (buf[j][i] != expect+10) {
+                printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                       __FILE__,__LINE__,j,i,expect+10,buf[j][i]);
                 nerrs++;
             }
     }
@@ -156,28 +222,57 @@ int main(int argc, char **argv) {
                              array_of_starts, MPI_ORDER_C,
                              MPI_INT, &buftype);
     MPI_Type_commit(&buftype);
-    err = ncmpi_get_vara_all(ncid, varid, start, count, ncbuf, 1, buftype); CHECK_ERR
+    err = ncmpi_get_vara_all(ncid, varid1, start, count, ncbuf, 1, buftype); CHECK_ERR
 
-    for (j=0; j<count[0]; j++) {
-        for (i=0; i<count[1]; i++)
-            if (buf[j][i] != ncbuf[(j+2)*(count[1]+4)+(i+2)]) {
-                printf("Error at line %d in %s: expecting ncbuf[%d][%d]=%d but got %d\n",
-                       __LINE__,__FILE__,j,i,buf[j][i],ncbuf[(j+2)*(count[1]+4)+(i+2)]);
-                nerrs++;
-            }
+    for (j=0; j<count[0]; j++) for (i=0; i<count[1]; i++) {
+        int getValue = ncbuf[(j+2)*(count[1]+4)+(i+2)];
+        if (buf[j][i] != getValue) {
+            printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                   __FILE__,__LINE__,j,i,buf[j][i],getValue);
+            nerrs++;
+        }
     }
+
+    for (i=0; i<(count[0]+4)*(count[1]+4); i++) ncbuf[i] = -1;
+    err = ncmpi_get_vara_all(ncid, varid2, start, count, ncbuf, 1, buftype); CHECK_ERR
+
+    for (j=0; j<count[0]; j++) for (i=0; i<count[1]; i++) {
+        int getValue = ncbuf[(j+2)*(count[1]+4)+(i+2)];
+        if (buf[j][i] != getValue) {
+            printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                   __FILE__,__LINE__,j,i,buf[j][i],getValue);
+            nerrs++;
+        }
+    }
+
     for (i=0; i<(count[0]+4)*(count[1]+4); i++) ncbuf[i] = -1;
 
-    err = ncmpi_iget_vara(ncid, varid, start, count, ncbuf, 1, buftype, &req); CHECK_ERR
+    err = ncmpi_iget_vara(ncid, varid1, start, count, ncbuf, 1, buftype, &req); CHECK_ERR
     err = ncmpi_wait_all(ncid, 1, &req, &st); CHECK_ERR
+    err = st; CHECK_ERR
 
-    for (j=0; j<count[0]; j++) {
-        for (i=0; i<count[1]; i++)
-            if (buf[j][i] != ncbuf[(j+2)*(count[1]+4)+(i+2)]) {
-                printf("Error at line %d in %s: expecting ncbuf[%d][%d]=%d but got %d\n",
-                       __LINE__,__FILE__,j,i,buf[j][i],ncbuf[(j+2)*(count[1]+4)+(i+2)]);
-                nerrs++;
-            }
+    for (j=0; j<count[0]; j++) for (i=0; i<count[1]; i++) {
+        int getValue = ncbuf[(j+2)*(count[1]+4)+(i+2)];
+        if (buf[j][i] != getValue) {
+            printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                   __FILE__,__LINE__,j,i,buf[j][i],getValue);
+            nerrs++;
+        }
+    }
+
+    for (i=0; i<(count[0]+4)*(count[1]+4); i++) ncbuf[i] = -1;
+
+    err = ncmpi_iget_vara(ncid, varid2, start, count, ncbuf, 1, buftype, &req); CHECK_ERR
+    err = ncmpi_wait_all(ncid, 1, &req, &st); CHECK_ERR
+    err = st; CHECK_ERR
+
+    for (j=0; j<count[0]; j++) for (i=0; i<count[1]; i++) {
+        int getValue = ncbuf[(j+2)*(count[1]+4)+(i+2)];
+        if (buf[j][i] != getValue) {
+            printf("Error at %s line %d: expect buf[%d][%d]=%d but got %d\n",
+                   __FILE__,__LINE__,j,i,buf[j][i],getValue);
+            nerrs++;
+        }
     }
 
     MPI_Type_free(&buftype);
@@ -193,6 +288,7 @@ int main(int argc, char **argv) {
         if (rank == 0 && sum_size > 0)
             printf("heap memory allocated by PnetCDF internally has %lld bytes yet to be freed\n",
                    sum_size);
+        if (malloc_size > 0) ncmpi_inq_malloc_list();
     }
 
     MPI_Allreduce(MPI_IN_PLACE, &nerrs, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
