@@ -2,7 +2,7 @@
  *  Copyright (C) 2014, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  *
- *  $Id: record.c 3341 2017-08-25 22:29:08Z wkliao $
+ *  $Id$
  */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
@@ -287,9 +287,11 @@ int main(int argc, char** argv) {
     /* check if PnetCDF freed all internal malloc */
     MPI_Offset malloc_size;
     err = ncmpi_inq_malloc_size(&malloc_size);
-    if (err == NC_NOERR && malloc_size > 0) /* this test is for running 1 process */
+    if (err == NC_NOERR && malloc_size > 0) { /* this test is for running 1 process */
         printf("heap memory allocated by PnetCDF internally has %lld bytes yet to be freed\n",
                malloc_size);
+        ncmpi_inq_malloc_list();
+    }
 
 fn_exit:
     MPI_Allreduce(MPI_IN_PLACE, &nerrs, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
